from espace import *
from math import *
from random import uniform

import entities
from weapons import Blaster, BlasterProjectile

def init():
  EnemyBlaster.model = Model.acquire("models/weapons/blaster.md3")
  EnemyBlaster.shader = Shader.acquireHandle("simple:models/weapons/blaster.jpg")
  EnemyBlaster.soundSources = list()
  for a in range(0, 4):
    source = SoundSource()
    source.setReferenceDistance(50)
    EnemyBlaster.soundSources.append(source)
  EnemyBlaster.soundSource = 0
  EnemyBlaster.lastSound = 0

class EnemyBlaster:
  def __init__(self, origin, angle):
    self.origin = origin
    self.angle = angle
    self.radius = 500
    self.spawntime = 0
    self.lastAttack = 0

  def think(self, deltaTime):
    pass

  def hit(self, player):
    time = System.time()
    if time - self.lastAttack < 0.75:
      return
    self.lastAttack = time

    dir = player.rd.origin - self.origin + Vector3(uniform(-200, 200), uniform(-200, 200), uniform(-200, 200))
    dir.normalize()
    entities.append(BlasterProjectile(self.origin, dir, None))

    if time - self.lastSound > 0.25:
      self.lastSound = time
      source = EnemyBlaster.soundSources[EnemyBlaster.soundSource]
      source.stop()
      source.setGain(0.02)
      source.play(Blaster.attack)

      EnemyBlaster.soundSource += 1
      if EnemyBlaster.soundSource == len(EnemyBlaster.soundSources):
        EnemyBlaster.soundSource = 0

  def addToScene(self):
    ent = RefEntity()

    ent.type = RefEntity.Model
    ent.renderfx = 0
    ent.modelHandle = EnemyBlaster.model
    ent.origin = self.origin
    ent.axis.identity()
    ent.axis.rotate(Vector3(0, 0, 1), acos(0) + self.angle / 90 * acos(0))
    ent.axis.rotate(Vector3(1, 0, 0), acos(0))
    ent.axis.scale(0.4, 0.4, 0.4)
    ent.frame = 0
    ent.skinNum = 0
    ent.customSkin = 0
    ent.customShader = EnemyBlaster.shader
    ent.color = Color(255, 255, 255)

    Renderer.appendRefEntity(ent)

