from espace import *
import random
from math import sin, cos, fmod

from path import Path
import enemies
import entities
import effects
from game import Game

quit = False

beat = 16412.0 / 44100.0

from intro import beat

class Container:
  pass

def play():
  Input.unbindAll()

  Input.bind(Input.K_ESCAPE, "eval intro.quit = True")
  Input.bind(ord('`'), "toggleconsole")
  Input.bind(ord('~'), "toggleconsole")

  splash = Splash(beat * 4, beat * 4)
  hallway01 = Hallway01()
  earth01 = Earth01()
  earth02 = Earth02()
  earth03 = Earth03()
  montages = Montages()
  faceball01 = Faceball01()
  faceball02 = Faceball02()
  rockets = Rockets()
  subtitles = Subtitles()
  fade = Fade()
  credits = Credits()
  outro = Outro()

  global quit

  backgroundTrack = SoundSource()
  backgroundStream = Media.acquire("intro/music.ogg")
  backgroundTrack.play(backgroundStream)

  startTime = System.time()

  lastTime = startTime

  while not quit:
    time = System.time() - startTime
    deltaTime = time - lastTime
    lastTime += deltaTime

    Shader.setTime(time)

    API.flushCommands()

    if True:
      if time < 128 * beat:
        earth01.render(time)
      elif time < 136 * beat:
        splash.render(time - 128 * beat, deltaTime)
      elif time < 200 * beat:
        credits.render(time - 136 * beat)
      elif time < 232 * beat:
        earth02.render(time - 200 * beat)
      elif time < 264 * beat:
        earth03.render(time - 232 * beat)
      elif time < 288 * beat:
        faceball01.render(time - 264 * beat)
      elif time < 352 * beat:
        faceball02.render(time - 288 * beat)
      elif time < 384 * beat:
        rockets.render(time - 352 * beat, deltaTime)
      elif time < 392 * beat:
        hallway01.render(time - 384 * beat, deltaTime)
      elif time < 456 * beat:
        outro.render(time - 392 * beat)
      else:
        break

      if time > 32 * beat:
        if time < 48 * beat:
          montages.render(0, time - 32 * beat)
        elif time < 64 * beat:
          montages.render(1, time - 48 * beat)
        elif time < 80 * beat:
          montages.render(2, time - 64 * beat)
        elif time < 96 * beat:
          montages.render(3, time - 80 * beat)
        elif time < 112 * beat:
          montages.render(4, time - 96 * beat)
        elif time < 128 * beat:
          montages.render(5, time - 112 * beat)

      if time < 16 * beat:
        fade.render(1 - time / (16 * beat))

      if time > 455 * beat:
        fade.render((time - 455 * beat) / beat)

      if time > 18 * beat and time < 128 * beat:
        subtitles.render(time - 18 * beat)

    Sound.idle()

    System.idle()

def simpleRefDef():
  ret = RefDef()
  ret.fovX = 45
  ret.fovY = GL.config().height * ret.fovX / GL.config().width
  ret.setViewport(0, 0, GL.config().width, GL.config().height)
  return ret

def simpleModelRefEnt(model):
  ret = RefEntity()
  ret.type = RefEntity.Model
  ret.renderfx = 0
  ret.modelHandle = model
  ret.axis.identity()
  ret.origin = Vector3(0, 0, 0)
  ret.frame = 0
  ret.skinNum = 0
  ret.customSkin = 0
  ret.customShader = 0
  ret.color = Color(255, 255, 255)
  return ret

class Splash:
  def __init__(self, sustain, decay):
    image = Image.acquireSimple("intro/logo.png")
    self.montage = Montage(image)
    self.sustain = sustain
    self.decay = decay
    self.exploded = False

  def render(self, time, deltaTime):
    GL.clear(GL.COLOR_BUFFER_BIT)

    if not self.exploded:
      self.exploded = True
      effects.addExplosion(Vector3(0, 0, 0))

    Shader.set3DMode()

    Renderer.clearScene()
    effects.think(deltaTime * 0.5)
    effects.addToScene()

    rd = simpleRefDef()
    rd.origin = Vector3(0, -300, 0)
    rd.axis.identity()
    rd.axis.lookAt(rd.origin, Vector3(0, 0, 0), Vector3(0, 0, 1))

    Renderer.renderScene(rd, None)

    Shader.set2DMode()

    GL.matrixMode(GL.PROJECTION)
    GL.pushMatrix()
    GL.loadIdentity()
    GL.ortho(0, 640, 480, 0, 0, 1);

    if time < self.sustain:
       Renderer.setColor(Color(255, 255, 255))
    else:
       i = 1 - ((time - self.sustain) / self.decay)
       Renderer.setColor(Color(i, i, i))

    self.montage.render(0, 0)

    GL.popMatrix()

class Earth01:
  def __init__(self):
    self.model = Model.acquire("intro/sphere.md3")
    self.shader = Shader.acquireHandle("intro/earth")
    self.glow = Shader.acquire("intro/earthglow")
    self.starfield = Montage(Image.acquireSimple("intro/starfield.png"))

  def render(self, time):
    GL.clear(GL.COLOR_BUFFER_BIT | GL.DEPTH_BUFFER_BIT)
    Shader.set2DMode()
    GL.matrixMode(GL.PROJECTION)
    GL.pushMatrix()
    GL.loadIdentity()
    GL.ortho(0, 640, 480, 0, 0, 1);
  
    Renderer.setColor(Color(255, 255, 255))
    self.starfield.render(sin(time) * 5, cos(time) * 5)

    Renderer.drawQuad2D(-100, -150, 1200, 1200, self.glow)

    GL.popMatrix()

    Shader.set3DMode()

    Renderer.clearScene()

    entity = simpleModelRefEnt(self.model)
    entity.customShader = self.shader
    entity.axis.scale(100, 100, 100)
    entity.axis.rotate(Vector3(0, 1, 0), 3.14 / 2)

    Renderer.appendRefEntity(entity)

    Renderer.addCorona(Vector3(1, 1, 0), Color(255, 240, 230))

    rd = simpleRefDef()

    rd.origin = Vector3(-8000, 4000, 5000)
    rd.axis.identity()

    Renderer.renderScene(rd)

class Earth02:
  def __init__(self):
    self.map = Map.acquire("intro/cutscene01.bsp")
    self.path0 = Path("intro/cutscene01_0.path")
    self.model = Model.acquire("models/ships/ship1.md3")
    self.skin = Shader.acquireHandle("simple:models/ships/ship1.png")
    self.blackbird = Montage(Image.acquireSimple("intro/blackbird.png"))
    self.mblackbird = Montage(Image.acquireSimple("intro/mblackbird.png"))
    self.redbird = Montage(Image.acquireSimple("intro/redbird.png"))
    self.mredbird = Montage(Image.acquireSimple("intro/mredbird.png"))

  def render(self, time):
    Shader.set3DMode()

    Renderer.clearScene()

    Renderer.addCorona(Vector3(0, -1, 0), Color(255, 240, 230))

    # Yes, mul with 1.7; cool side effect.
    t = time / (32 * beat + 0.1) * 1.7

    ent = RefEntity()
    ent.type = RefEntity.Model
    ent.renderfx = 0
    ent.modelHandle = self.model
    ent.origin = self.path0.point(t)
    target = self.path0.point(t + 0.01)
    ent.axis.identity()
    ent.axis.lookAt(ent.origin, target, Vector3(0, 0, -1))
    ent.axis.scale(-2, 2, -2)
    ent.frame = 0
    ent.skinNum = 0
    ent.customSkin = 0
    ent.customShader = self.skin
    ent.color = Color(255, 255, 255)

    Renderer.appendRefEntity(ent)

    rd = simpleRefDef()
    rd.origin = Vector3(-1000 + t * 1000, -4000, 1000)
    rd.axis.lookAt(rd.origin, self.path0.point(t), Vector3(0, 0, 1))
    rd.glFog.mode = GL.EXP
    rd.glFog.hint = GL.DONT_CARE
    rd.glFog.color = Color(64, 92, 128)
    rd.glFog.near = 0.75
    rd.glFog.far = 1.0
    rd.glFog.density = 0.0004

    GL.enable(GL.FOG)

    Renderer.renderScene(rd, self.map)

    GL.disable(GL.FOG)

    Shader.set2DMode()

    GL.matrixMode(GL.PROJECTION)
    GL.pushMatrix()
    GL.loadIdentity()
    GL.ortho(0, 640, 480, 0, 0, 1);

    bt = int(time / beat / 2)

    birds = (self.blackbird, self.mblackbird, self.mredbird, self.redbird)

    for i in range(0, bt % 4):
      birds = birds[1:] + birds[:1]

    birds[0].render(0, 480 - 200) 
    birds[1].render(100, 480 - 200) 
    birds[2].render(0, 480 - 100) 
    birds[3].render(100, 480 - 100) 

    GL.popMatrix()

class Earth03:
  def __init__(self):
    self.model = Model.acquire("intro/sphere.md3")
    self.shader = Shader.acquireHandle("intro/earth")
    self.glow = Shader.acquire("intro/earthglow")
    self.starfield = Montage(Image.acquireSimple("intro/starfield.png"))
    self.ship = Model.acquire("models/ships/ship1.md3")
    self.shipShader = Shader.acquireHandle("simple:models/ships/ship1.png")

  def render(self, time):
    GL.clear(GL.COLOR_BUFFER_BIT | GL.DEPTH_BUFFER_BIT)
    Shader.set2DMode()
    GL.matrixMode(GL.PROJECTION)
    GL.pushMatrix()
    GL.loadIdentity()
    GL.ortho(0, 640, 480, 0, 0, 1);
  
    Renderer.setColor(Color(255, 255, 255))
    self.starfield.render(sin(time) * 5, cos(time) * 5)

    Renderer.drawQuad2D(-100, -150, 1200, 1200, self.glow)

    GL.popMatrix()

    Shader.set3DMode()

    Renderer.clearScene()

    entity = simpleModelRefEnt(self.model)
    entity.customShader = self.shader
    entity.axis.scale(100, 100, 100)
    entity.axis.rotate(Vector3(0, 1, 0), 3.14 / 2)

    Renderer.appendRefEntity(entity)

    entity = simpleModelRefEnt(self.ship)
    entity.customShader = self.shipShader
    entity.origin = Vector3(-time * 700, time * 400, 5000)
    entity.axis.identity()
    entity.axis.rotate(Vector3(0, 0, 1), 12.028)

    Renderer.appendRefEntity(entity)

    Renderer.addCorona(Vector3(1, 1, 0), Color(255, 240, 230))

    rd = simpleRefDef()

    rd.origin = Vector3(-8000, 4000, 5000)
    rd.axis.identity()

    Renderer.renderScene(rd)

class Hallway01:
  def __init__(self):
    self.map = Map.acquire("intro/cutscene00.bsp")
    self.path0 = Path("intro/cutscene00_0.path")
    self.path1 = Path("intro/cutscene00_1.path")
    self.model = Model.acquire("models/ships/ship1.md3")
    self.skin = Shader.acquireHandle("simple:models/ships/ship1.png")
    self.entities = list()

    for i in range(0, self.map.entityCount()):
      entity = self.map.entity(i)
      classname = entity.get("classname")
      if classname == "enemy_blaster":
        origin = entity.get("origin").split()
        origin = Vector3(float(origin[0]), float(origin[1]), float(origin[2]))
        angle = entity.get("angle")

        if angle == None:
          angle = 0
        else:
          angle = float(angle)

        self.entities.append(enemies.EnemyBlaster(origin, angle))

  def render(self, time, deltaTime):
    Shader.set3DMode()
    GL.clear(GL.DEPTH_BUFFER_BIT)

    Game.currentMap = self.map

    t = time / (8 * beat + 0.2)

    Renderer.clearScene()

    ent = RefEntity()
    ent.type = RefEntity.Model
    ent.renderfx = 0
    ent.modelHandle = self.model
    ent.origin = self.path0.point(t)
    target = self.path0.point(t + 0.1)
    ent.axis.identity()
    ent.axis.rotate((target - ent.origin).normalize(), sin(time) * 1.5)
    ent.axis.lookAt(ent.origin, target, Vector3(0, 0, 1))
    ent.frame = 0
    ent.skinNum = 0
    ent.customSkin = 0
    ent.customShader = self.skin
    ent.color = Color(255, 255, 255)

    Renderer.appendRefEntity(ent)

    player = Container()
    player.rd = Container()
    player.rd.origin = ent.origin
    player.radius = 50

    for e in self.entities:
      if e.spawntime == 0:
        e.addToScene()
        if (e.origin - ent.origin).magnitude() < 800:
          e.hit(player)

    entities.think(deltaTime)
    entities.addToScene()

    rd = simpleRefDef()
    rd.origin = self.path1.point(t)
    rd.axis.lookAt(rd.origin, self.path0.point(t), Vector3(0, 0, 1))

    direction = Vector3(rd.axis(0, 0), rd.axis(1, 0), rd.axis(2, 0))
    up = Vector3(rd.axis(0, 2), rd.axis(1, 2), rd.axis(2, 2))

    Sound.setPosition(rd.origin)
    Sound.setOrientation(direction, up)

    Renderer.renderScene(rd, self.map)

class Montages:
  def __init__(self):
    self.montages = list()
    self.montages.append(Montage(Image.acquireSimple("intro/armyman.png")))
    self.montages.append(Montage(Image.acquireSimple("intro/cannon.png")))
    self.montages.append(Montage(Image.acquireSimple("intro/dead.png")))
    self.montages.append(Montage(Image.acquireSimple("intro/linje.png")))
    self.montages.append(Montage(Image.acquireSimple("intro/people.png")))
    self.montages.append(Montage(Image.acquireSimple("intro/tanks.png")))

  def render(self, number, time):
    Shader.set2DMode()
    GL.matrixMode(GL.PROJECTION)
    GL.pushMatrix()
    GL.loadIdentity()
    GL.ortho(0, 640, 480, 0, 0, 1);

    if time < 1.0:
      Renderer.setColor(Color(1.0, 1.0, 1.0, time))

      for i in range(0, 10):
        self.montages[number].render((i - 5) * 20 * (1 - time), 0)
    elif time < beat * 15:
      Renderer.setColor(Color(255, 255, 255))
      for i in range(0, 10):
        self.montages[number].render(0, 0)
    else:
      t = (time - beat * 15) / beat

      Renderer.setColor(Color(1.0, 1.0, 1.0, 1 - t))

      for i in range(0, 11):
        self.montages[number].render(0, (i - 5) * 20 * t)

    GL.popMatrix()

class Subtitles:
  text = "It is the year 10191.  Massive wars have reigned the Earth for hundreds of millions of billions of zigadillions of years.  The uncontrolled use of natural resources, has driven the nations of the planet against each other in a fight for the last remaining necessities of survival. Due to the colossal increasing use of conventional, atomic, biological and chemical weapons, and pop music, the world is soon to be left in a state of total eradication.  A small idealist group based in the caves of Western Canada, has spent the last century planning a search for a new planet to settle on.  One man is now boldly setting out to find this new world, rich of energy, minerals and hot potatoes."#.upper()

  def __init__(self):
    self.font = Font()
    Renderer.registerFont("fonts/intro_64.dat", self.font)

  def render(self, time):
    Shader.set2DMode()
    GL.matrixMode(GL.PROJECTION)
    GL.pushMatrix()
    GL.loadIdentity()
    GL.ortho(0, 1024, 768, 0, 0, 1);

    Renderer.setColor(Color(0, 0, 0, 1.0))
    Renderer.put(1029 - time * 650, 685, self.font, Subtitles.text)
    Renderer.setColor(Color(0, 0, 0, 1.0))
    Renderer.put(1019 - time * 650, 675, self.font, Subtitles.text)
    Renderer.setColor(Color(0.125, 1, 0.25, 0.75))
    Renderer.put(1024 - time * 650, 680, self.font, Subtitles.text)

    GL.loadIdentity()
    GL.ortho(0, 640, 480, 0, 0, 1);
    Renderer.setColor(Color(1, 1, 1, 0.125))
    Renderer.put(640 - time * 650, 480, self.font, Subtitles.text)
    GL.loadIdentity()
    GL.ortho(0, 4096, 3072, 0, 0, 1);
    Renderer.put(4096 - time * 650, 2500, self.font, Subtitles.text)

    GL.popMatrix()

class Faceball01:
  def __init__(self):
    self.model = Model.acquire("intro/faceball.md3")
    self.shader = Shader.acquireHandle("intro/face")
    self.cartoon = Montage(Image.acquireSimple("intro/cartoon.png"))

  def render(self, time):
    Shader.set3DMode()
    GL.clear(GL.COLOR_BUFFER_BIT)

    Renderer.clearScene()

    for i in range(0, 20):
      ent = simpleModelRefEnt(self.model)
      ent.customShader = self.shader
      x = (-100 + i * 10) * (1 + 0.4 * sin(time * 8))
      y = cos(i * 1.0 + time * 3) * 90 + sin(time * 4) * 30
      (x, y) = (cos(time) * x + sin(time * 3) * y,
                cos(time * 2) * y - sin(time) * x)

      ent.origin = Vector3(x, 0, y)
      ent.axis.rotate(Vector3(0, 1, 0), i + time)
      ent.color = Color(192, 192, 192)

      Renderer.appendRefEntity(ent)

    rd = simpleRefDef()

    rd.origin = Vector3(0, 100, 0)
    rd.axis.identity()
    rd.axis.lookAt(rd.origin, Vector3(0, 0, 0), Vector3(0, 0, 1))

    Renderer.renderScene(rd)

    Shader.set2DMode()

    bt = fmod(time / beat, 4)

    if 1 < bt < 2 or 2.5 < bt < 3.5:
      x = 40
    else:
      x = -40

    GL.matrixMode(GL.PROJECTION)
    GL.pushMatrix()
    GL.loadIdentity()
    GL.ortho(x, 640 - x, 480 - x, x, 0, 1);

    Renderer.setColor(Color(255, 255, 255))

    self.cartoon.render(0, 0)

    GL.matrixMode(GL.PROJECTION)
    GL.popMatrix()

class Faceball02:
  def __init__(self):
    self.model = Model.acquire("intro/faceball.md3")
    self.shader = Shader.acquireHandle("intro/face")
    self.shader2 = Shader.acquireHandle("intro/faceopaque")
    self.map = Map.acquire("intro/cutscene02.bsp")
    self.which = Montage(Image.acquireSimple("intro/which.png"))
    self.you = Montage(Image.acquireSimple("intro/you.png"))
    self.army1 = Montage(Image.acquireSimple("intro/army1.png"))
    self.army2 = Montage(Image.acquireSimple("intro/army2.png"))
    self.doyou = Montage(Image.acquireSimple("intro/do_you.png"))

  def render(self, time):
    Shader.set3DMode()
    GL.clear(GL.COLOR_BUFFER_BIT)

    Renderer.clearScene()

    for i in range(0, 10):
      ent = simpleModelRefEnt(self.model)
      ent.origin = Vector3(0, 0, -time * 750 + i * 10)
      if i > 0:
        ent.customShader = self.shader
        c = (1 - i / 9.0)
        ent.color = Color(c, c, c)
      else:
        ent.customShader = self.shader2

      Renderer.appendRefEntity(ent)

    rd = simpleRefDef()

    rd.origin = Vector3(100 * sin(time * 2),
                        100 * cos(time * 2),
                        -time * 750 + sin(time) * 200)
    rd.axis.identity()
    rd.axis.lookAt(rd.origin, Vector3(0, 0, -time * 750), Vector3(0, 0, 1))

    rd.glFog.mode = GL.EXP
    rd.glFog.hint = GL.DONT_CARE
    rd.glFog.color = Color(255, 255, 255)
    rd.glFog.near = 0.75
    rd.glFog.far = 1.0
    rd.glFog.density = 0.0003

    GL.enable(GL.FOG)

    Renderer.renderScene(rd, self.map)

    GL.disable(GL.FOG)

    measure = int(time / beat / 4)

    bt = fmod(time / beat, 4)

    Shader.set2DMode()

    GL.matrixMode(GL.PROJECTION)
    GL.pushMatrix()
    GL.loadIdentity()
    GL.ortho(0, 640, 480, 0, 0, 1);

    Renderer.setColor(Color(255, 255, 255))

    if 1 < bt < 2:
      s = 2 - bt
      Renderer.setColor(Color(1, 1, 1, 1.0))
    elif 2.5 < bt < 3.5:
      s = 4.5 - bt
      Renderer.setColor(Color(1, 1, 1, 1.0))
    else:
      Renderer.setColor(Color(1, 1, 1, 0.0))

    if measure == 2 or measure == 6:
      self.which.render(random.uniform(-5, 5), random.uniform(-5, 5))
    elif measure == 3 or measure == 7:
      self.you.render(random.uniform(-5, 5), random.uniform(-5, 5))
    elif measure == 4 or measure == 8:
      self.army1.render(random.uniform(-5, 5), random.uniform(-5, 5))
    elif measure == 5 or measure == 9:
      self.army2.render(random.uniform(-5, 5), random.uniform(-5, 5))
    elif measure >= 10:
      self.doyou.render(random.uniform(-5, 5), random.uniform(-5, 5))

    GL.matrixMode(GL.PROJECTION)
    GL.popMatrix()

class Rockets:
  def __init__(self):
    self.map = Map.acquire("intro/cutscene03.bsp")
    self.path0 = Path("intro/cutscene03_0.path")
    self.path1 = Path("intro/cutscene03_1.path")
    self.path2 = Path("intro/cutscene03_2.path")
    self.path3 = Path("intro/cutscene03_3.path")
    self.model = Model.acquire("models/ships/ship1.md3")
    self.shader = Shader.acquireHandle("simple:models/ships/ship1.png")
    self.face = Model.acquire("intro/faceball.md3")
    self.faceShader = Shader.acquireHandle("intro/face")
    self.faceShader2 = Shader.acquireHandle("intro/faceopaque")
    self.rocket = Model.acquire("models/weapons/rocket.md3")
    self.rocketShader = Shader.acquireHandle("simple:models/weapons/rocket.png")
    self.exploded = [False, False]

  def render(self, time, deltaTime):
    Shader.set3DMode()
    GL.clear(GL.COLOR_BUFFER_BIT)

    Renderer.clearScene()

    if time < 2:
      for i in range(0, 10):
        ent = simpleModelRefEnt(self.face)
        ent.origin = Vector3(0, 0, 200 - time * 350 + i * 10)
        if i > 0:
          ent.customShader = self.faceShader
          c = (1 - i / 9.0)
          ent.color = Color(c, c, c)
        else:
          ent.customShader = self.faceShader2

        Renderer.appendRefEntity(ent)

    rd = simpleRefDef()

    t = time / (32 * beat + 0.1)

    entity = simpleModelRefEnt(self.model)
    entity.customShader = self.shader
    entity.origin = self.path0.point(t)
    target = self.path0.point(t + 0.01)
    entity.axis.identity()
    entity.axis.lookAt(entity.origin, target, Vector3(0, 0, -1))
    entity.axis.scale(-1.5, 1.5, -1.5)

    Renderer.appendRefEntity(entity)

    if t < 0.92:
      t2 = t / 0.92 * 0.99
      entity = simpleModelRefEnt(self.rocket)
      entity.customShader = self.rocketShader
      entity.origin = self.path2.point(t2)
      target2 = self.path2.point(t2 + 0.01)
      entity.axis.identity()
      entity.axis.lookAt(entity.origin, target2, Vector3(0, 0, -1))
      entity.axis.scale(-1, 1, -1)
      dir = target2 - entity.origin
      dir.normalize()
      effects.addFire(entity.origin - dir * 70)
      effects.addSmoke(entity.origin - dir * 70)
      self.exploded[0] = False

      Renderer.appendRefEntity(entity)
    elif not self.exploded[0]:
      effects.addExplosion(self.path2.point(1))
      self.exploded[0] = True

    if t < 0.2:
      pass
    elif t < 0.7:
      t2 = (t - 0.2) / 0.5 * 0.99
      entity = simpleModelRefEnt(self.rocket)
      entity.customShader = self.rocketShader
      entity.origin = self.path3.point(t2)
      target2 = self.path3.point(t2 + 0.01)
      entity.axis.identity()
      entity.axis.lookAt(entity.origin, target2, Vector3(0, 0, -1))
      entity.axis.scale(-1, 1, -1)
      dir = target2 - entity.origin
      dir.normalize()
      effects.addFire(entity.origin - dir * 70)
      effects.addSmoke(entity.origin - dir * 70)
      self.exploded[1] = False

      Renderer.appendRefEntity(entity)
    elif not self.exploded[1]:
      effects.addExplosion(self.path3.point(1))
      self.exploded[1] = True

    effects.think(deltaTime)
    effects.addToScene()

    rd.origin = self.path1.point(t)
    rd.axis.identity()
    rd.axis.lookAt(rd.origin, target, Vector3(0, 0, 1))

    direction = Vector3(rd.axis(0, 0), rd.axis(1, 0), rd.axis(2, 0))
    up = Vector3(rd.axis(0, 2), rd.axis(1, 2), rd.axis(2, 2))

    Sound.setPosition(rd.origin)
    Sound.setOrientation(direction, up)

    Renderer.renderScene(rd, self.map)

class Fade:
  def __init__(self):
    self.shader = Shader.acquire("simple:*white")

  def render(self, time):
    Shader.set2DMode()
    
    Renderer.setColor(Color(0, 0, 0, time))
    Renderer.drawQuad2D(0, 0, GL.config().width, GL.config().height, self.shader)

class Credits:
  def __init__(self):
    self.horiz = Montage(Image.acquireSimple("intro/scroller.png"))
    self.vert = Montage(Image.acquireSimple("intro/filmscroll.png"))
    self.jorgen = list()
    self.jorgen.append(Montage(Image.acquireSimple("intro/jorgen0.jpg")))
    self.jorgen.append(Montage(Image.acquireSimple("intro/jorgen1.jpg")))
    self.jorgen.append(Montage(Image.acquireSimple("intro/jorgen2.jpg")))
    self.jorgen.append(Montage(Image.acquireSimple("intro/jorgen3.jpg")))
    self.jorgen.append(self.jorgen[2])
    self.jorgen.append(self.jorgen[1])
    self.morten = list()
    self.morten.append(Montage(Image.acquireSimple("intro/morten0.jpg")))
    self.morten.append(Montage(Image.acquireSimple("intro/morten1.jpg")))
    self.morten.append(Montage(Image.acquireSimple("intro/morten2.jpg")))
    self.morten.append(Montage(Image.acquireSimple("intro/morten3.jpg")))
    self.morten.append(self.morten[2])
    self.morten.append(self.morten[1])
    self.shane = Montage(Image.acquireSimple("intro/shane.png"))
    self.sirkel = Montage(Image.acquireSimple("intro/sirkel.png"))
    self.envobj = Model.acquire("intro/envobj2.md3")
    self.envmap = Shader.acquireHandle("intro/envmap")

  def render(self, time):
    Shader.set2DMode()

    Renderer.clearScene()

    Renderer.setColor(Color(255, 255, 255))

    GL.matrixMode(GL.PROJECTION)
    GL.pushMatrix()
    GL.loadIdentity()
    GL.ortho(0, 640, 480, 0, 0, 1);

    if time < 32 * beat:
      t = int(time * 8) % len(self.jorgen)

      self.jorgen[t].render(0, 0)
    else:
      t = int(time * 8) % len(self.morten)

      self.morten[t].render(0, 0)

    GL.loadIdentity()
    GL.ortho(0, 640, 640, 0, 0, 1);

    self.horiz.render(-640 * (time / (64 * beat)), 0)

    GL.matrixMode(GL.PROJECTION)
    GL.popMatrix()

    Shader.set3DMode()

    ent = simpleModelRefEnt(self.envobj)
    ent.origin = Vector3(0, 0, 0)
    ent.axis.identity()
    ent.axis.rotate(Vector3(cos(time), 0, sin(time)), time)

    bt = fmod(time / beat, 4)

    if 1 < bt < 2:
      s = 3 - bt
      ent.axis.scale(s, s, s)
    elif 2.5 < bt < 3.5:
      s = 4.5 - bt
      ent.axis.scale(s, s, s)

    ent.customShader = self.envmap

    Renderer.appendRefEntity(ent)

    rd = simpleRefDef()
    rd.origin = Vector3(0, -400, 0)
    rd.axis.identity()
    
    rd.axis.lookAt(rd.origin, Vector3(200, 0, 0), Vector3(0, 0, 1))

    Renderer.renderScene(rd, None)

    Shader.set2DMode()

    if time > 16 * beat and time < 32 * beat:
      GL.matrixMode(GL.PROJECTION)
      GL.pushMatrix()
      GL.loadIdentity()
      GL.ortho(0, 640, 480, 0, 0, 1);

      self.shane.render(0, 0)
      GL.matrixMode(GL.PROJECTION)
      GL.popMatrix()

    if time > 46 * beat and time < 64 * beat:
      GL.matrixMode(GL.PROJECTION)
      GL.pushMatrix()
      GL.loadIdentity()
      GL.ortho(0, 640, 480, 0, 0, 1);

      self.sirkel.render(0, 0)
      GL.matrixMode(GL.PROJECTION)
      GL.popMatrix()

class Outro:
  def __init__(self):
    self.teddy = Montage(Image.acquireSimple("intro/teddy.png"))
    self.pingvin = Montage(Image.acquireSimple("intro/pingvin.png"))
    self.last = Montage(Image.acquireSimple("intro/siste.png"))

  def render(self, time):
    GL.clear(GL.COLOR_BUFFER_BIT)

    Shader.set2DMode()

    GL.matrixMode(GL.PROJECTION)
    GL.pushMatrix()
    GL.loadIdentity()
    GL.ortho(0, 640, 480, 0, 0, 1);

    Renderer.setColor(Color(255, 255, 255))

    if time < 16 * beat:
      self.teddy.render(0, 0)
    elif time < 32 * beat:
      self.pingvin.render(0, 0)
    else:
      self.last.render(0, 0)

    GL.matrixMode(GL.PROJECTION)
    GL.popMatrix()

