# The classes in this namespace define the look and behavior of powerups, i.e.
# hull repair kits, shield upgrades, ammunition and weapons.

# Import all classes from Empty Space
from espace import *

# The shader used for creating a colored glow around powerups.
glow = 0

def init():
  global glow
  glow = Shader.acquireHandle("textures/effects/glow")
  BasicShield.shader = Shader.acquireHandle("textures/powerups/bshield")
  NormalShield.shader = Shader.acquireHandle("textures/powerups/nshield")
  AdvancedShield.shader = Shader.acquireHandle("textures/powerups/ashield")
  RepairKit.shader = Shader.acquireHandle("textures/powerups/repairkit")
  Rockets.shader = Shader.acquireHandle("textures/powerups/rockets")
  EnergyPack.shader = Shader.acquireHandle("textures/powerups/energy")
  Slugs.shader = Shader.acquireHandle("textures/powerups/slugs")
  RocketLauncher.shader = Shader.acquireHandle("textures/powerups/rocketlauncher")
  Storm.shader = Shader.acquireHandle("textures/powerups/storm")
  RailGun.shader = Shader.acquireHandle("textures/powerups/railgun")

# Add a glow effect to the render list.  The type parameter determines the color
# of the glow, to make it possible to have different colors on the different
# classes of powerups.
def addGlow(origin, type):
  global glow

  entity = RefEntity()

  entity.type = RefEntity.Sprite
  entity.renderfx = 0
  entity.origin = origin
  entity.customShader = glow
  entity.radius = 50
  if type == 0:
    entity.color = Color(127, 127, 91)
  elif type == 1:
    entity.color = Color(91, 127, 127)
  else:
    entity.color = Color(127, 91, 127)

  Renderer.appendRefEntity(entity)

class BasicShield:
  def __init__(self, origin):
    self.origin = origin
    self.radius = 20
    self.spawntime = 0

  def think(self, deltaTime):
    pass

  def hit(self, player):
    if player.shield < 50:
      player.shield = 50
      self.spawntime = 30 

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.Sprite
    entity.render = 0
    entity.origin = self.origin
    entity.customShader = BasicShield.shader
    entity.radius = self.radius
    entity.rotation = 0

    Renderer.prependRefEntity(entity)

    addGlow(self.origin, 0)

class NormalShield:
  def __init__(self, origin):
    self.origin = origin
    self.radius = 20
    self.spawntime = 0

  def think(self, deltaTime):
    pass

  def hit(self, player):
    if player.shield < 100:
      player.shield = 100
      self.spawntime = 30

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.Sprite
    entity.render = 0
    entity.origin = self.origin
    entity.customShader = NormalShield.shader
    entity.radius = self.radius
    entity.rotation = 0

    Renderer.prependRefEntity(entity)

    addGlow(self.origin, 0)

class AdvancedShield:
  def __init__(self, origin):
    self.origin = origin
    self.radius = 20
    self.spawntime = 0

  def think(self, deltaTime):
    pass

  def hit(self, player):
    if player.shield < 200:
      player.shield = 200
      self.spawntime = 30

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.Sprite
    entity.render = 0
    entity.origin = self.origin
    entity.customShader = AdvancedShield.shader
    entity.radius = self.radius
    entity.rotation = 0

    Renderer.prependRefEntity(entity)

    addGlow(self.origin, 0)

class RepairKit:
  def __init__(self, origin):
    self.origin = origin
    self.radius = 20
    self.spawntime = 0

  def think(self, deltaTime):
    pass

  def hit(self, player):
    if player.hull < 100:
      if player.hull > 80:
        player.hull = 100
      else:
        player.hull += 20
      self.spawntime = 30

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.Sprite
    entity.render = 0
    entity.origin = self.origin
    entity.customShader = RepairKit.shader
    entity.radius = self.radius
    entity.rotation = 0

    Renderer.prependRefEntity(entity)

    addGlow(self.origin, 0)

class Rockets:
  def __init__(self, origin):
    self.origin = origin
    self.radius = 20
    self.spawntime = 0

  def think(self, deltaTime):
    pass

  def hit(self, player):
    player.rockets += 5
    self.spawntime = 30

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.Sprite
    entity.render = 0
    entity.origin = self.origin
    entity.customShader = Rockets.shader
    entity.radius = self.radius
    entity.rotation = 0

    Renderer.prependRefEntity(entity)

    addGlow(self.origin, 1)

class EnergyPack:
  def __init__(self, origin):
    self.origin = origin
    self.radius = 20
    self.spawntime = 0

  def think(self, deltaTime):
    pass

  def hit(self, player):
    player.energy += 10
    self.spawntime = 30

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.Sprite
    entity.render = 0
    entity.origin = self.origin
    entity.customShader = EnergyPack.shader
    entity.radius = self.radius
    entity.rotation = 0

    Renderer.prependRefEntity(entity)

    addGlow(self.origin, 1)

class Slugs:
  def __init__(self, origin):
    self.origin = origin
    self.radius = 20
    self.spawntime = 0

  def think(self, deltaTime):
    pass

  def hit(self, player):
    player.slugs += 5
    self.spawntime = 30

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.Sprite
    entity.render = 0
    entity.origin = self.origin
    entity.customShader = Slugs.shader
    entity.radius = self.radius
    entity.rotation = 0

    Renderer.prependRefEntity(entity)

    addGlow(self.origin, 1)

class RocketLauncher:
  def __init__(self, origin):
    self.origin = origin
    self.radius = 20
    self.spawntime = 0

  def think(self, deltaTime):
    pass

  def hit(self, player):
    player.rocketLauncher = True
    player.rockets += 5
    self.spawntime = 30

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.Sprite
    entity.render = 0
    entity.origin = self.origin
    entity.customShader = RocketLauncher.shader
    entity.radius = self.radius
    entity.rotation = 0

    Renderer.prependRefEntity(entity)

    addGlow(self.origin, 2)

class Storm:
  def __init__(self, origin):
    self.origin = origin
    self.radius = 20
    self.spawntime = 0

  def think(self, deltaTime):
    pass

  def hit(self, player):
    player.storm = True
    player.energy += 10
    self.spawntime = 30

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.Sprite
    entity.render = 0
    entity.origin = self.origin
    entity.customShader = Storm.shader
    entity.radius = self.radius
    entity.rotation = 0

    Renderer.prependRefEntity(entity)

    addGlow(self.origin, 2)

class RailGun:
  def __init__(self, origin):
    self.origin = origin
    self.radius = 20
    self.spawntime = 0

  def think(self, deltaTime):
    pass

  def hit(self, player):
    player.railGun = True
    player.slugs += 4
    self.spawntime = 30

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.Sprite
    entity.render = 0
    entity.origin = self.origin
    entity.customShader = RailGun.shader
    entity.radius = self.radius
    entity.rotation = 0

    Renderer.prependRefEntity(entity)

    addGlow(self.origin, 2)


