#ifndef MONTAGE_H_
#define MONTAGE_H_

#ifndef SWIG
#include "image.h"
#include "types.h"
#endif

/**
 * Image montage routines.
 *
 * OpenGL requires the width and height of textures to be in powers of two.
 * This class aids in displaying images not fitting this requirement, by
 * subdividing the images into smaller images.
 *
 * \author Morten Hustveit
 */
class IMPORT Montage
{
public:

  /**
   * Constructs a new image montage.
   *
   * \todo How about a const Image&?
   * \param image Image to read.
   */
  Montage(Image* image);

  /**
   * Destroys a montage image.
   */
  ~Montage();

  /**
   * Render the image montage at the given coordinates.
   */
  void render(float x = 0, float y = 0) const;

  /**
   * Sets the source blend factor.  Default is GL::SRC_ALPHA.
   */
  void setSourceBlend(uint factor);

  /**
   * Sets the destination blend factor.  Default is GL::ONE_MINUS_SRC_ALPHA.
   */
  void setDestBlend(uint factor);

protected:

  uint width, height;
  Image** images;
  uint sourceBlend, destBlend;
};

#endif // !MONTAGE_H_

// vim: ts=2 sw=2 et
