#ifndef OUTPUT_H_
#define OUTPUT_H_

#include <ostream>

#include "types.h"
/**
 * \file output.h
 * \brief Output stream declarations.
 *
 * There are four standard C++ ostreams provided for output:
 *  - esError
 *  - esWarning
 *  - esDebug
 *  - esInfo
 *
 * \todo Messages printed with esDebug can/should be disabled -- how?
 *
 * Messages printed using these streams are printed both in the Console and on
 * standard error.
 *
 * esError, esWarning and esDebug prepends new lines with `Error:', `Warning'
 * and `Debug:'.
 *
 * \see Console
 */

extern IMPORT std::ostream esError;   /**< Errors considered \b fatal. */
extern IMPORT std::ostream esWarning; /**< Abnormal events. */
extern IMPORT std::ostream esDebug;   /**< Debug messages. */
extern IMPORT std::ostream esInfo;    /**< Normal messages. */

#endif // !OUTPUT_H_

// vim: ts=2 sw=2 et
