#ifndef SYSTEM_H_
#define SYSTEM_H_

#ifndef SWIG
#include "types.h"
#endif

/**
 * Platform specific routines.
 * \author Morten Hustveit
 */
struct System
{
  /**
   * Initialize system specific structures.
   *
   * \todo Make one initialization function for each subsystem, and make the
   * different subsystem's initialization routines call them implicitly.  Then
   * remove this function.
   */
  static IMPORT void initialize();

  /**
   * Swaps buffer and process platform specific events.
   */
  static IMPORT void idle();

  /**
   * Get time.
   * \return Seconds passed since a system specific time.
   */
  static IMPORT double time();

  /**
   * Check available memory.
   * \return Bytes of available physical memory.
   */
  static IMPORT uint memoryRemaining();

#ifndef SWIG
  /**
   * Open dynamic library.
   * \param fileName Name of dynamic library.
   * \return Dynamic libary handle.
   */
  static IMPORT void* dlopen(const char* fileName);
  /**
   * Get address of symbol in dynamic library.
   * \param handle Dynamic library handle.
   * \param symbol NULL-terminated symbol name.
   * \return NULL on failure, address on success.
   */
  static IMPORT void* dlsym(void* handle, char* symbol);
  /**
   * Get human readable error message.
   * \return Pointer to the last error message.
   */
  static IMPORT const char* dlerror();
  /**
   * Close dynamic library.
   * \param handle Dynamic library handle.
   */
  static IMPORT void dlclose(void* handle);

protected:

  friend class Renderer;

  static IMPORT void updateScreen();

#endif // !SWIG
};

#endif // !SYSTEM_H_

// vim: ts=2 sw=2 et
