#ifndef UTIL_H_
#define UTIL_H_

#include "types.h"

/**
 * The Util namespace provides a higher level interface to the different
 * subsystems.
 *
 * This should suit most needs at no additional cost, it will result in less
 * code and which means less to maintain - prefer this over manual interaction
 * with the subsystems.
*/
struct Util
{
  /**
   * Flags for fine grained control of which subsystems to initialize.
   * \see initialize()
   */
  enum Flags
  {
    NoShaders = 1, /**< Do not set up shader system. */
    NoAudio = 2,   /**< Do not set up audio system. */
    NoConsole = 4, /**< Do not render console in idle loop. */
  };

  /**
   * Initialize the Empty Space engine.
   *
   * An example of how commands can be passed on the command line:
   * <pre>./game +set r_mode 3 +bind Q quit</pre>
   *
   * If argc and argv is provided command line commands will be parsed
   * and executed, new commands must start with `+'.
   * The flags argument can be used for fine grained control of which
   * subsystems should be started.
   *
   * \param argc The count of elements in the argv array.
   * \param argv The array of parameters.
   * \param flags Bit mask with enum Flags.
   *
   * \see enum Flags
   */
  static IMPORT void initialize(int argc = 0, char** argv = 0, uint flags = 0);

  /**
   * The idle loop.
   * This will update the screen and audio and process commands.
   */
  static IMPORT void idle();
};

#endif // !UTIL_H_

// vim: ts=2 sw=2 et
