/***************************************************************************
                           output.cc  -  Classes for human readable output
                               -------------------
      copyright            : (C) 2003 by Morten Hustveit
      email                : morten@debian.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include <espace/output.h>
#include <espace/console.h>

class Stream : public std::streambuf
{
public:

  Stream(const char* prefix)
    : prefix(prefix),
      lineStart(true)
  {
    setp(0, 0);
  }

protected:

  int overflow(int c = EOF);

  const char* prefix;
  bool lineStart;
};

std::ostream esError(new Stream("Error: "));
std::ostream esWarning(new Stream("Warning: "));
std::ostream esDebug(new Stream("Debug: "));
std::ostream esInfo(new Stream(""));

int Stream::overflow(int c)
{
  if(lineStart)
  {
    lineStart = false;

    for(uint i = 0; i < strlen(prefix); ++i)
      overflow(prefix[i]);
  }

  if(c == '\n')
    lineStart = true;

  std::cerr.put(c);
  Console::put(c);

  return 0;
}

// vim: ts=2 sw=2 et
