%module espace
%{
#include <espace/api.h>
#include <espace/console.h>
#include <espace/cvar.h>
#include <espace/file.h>
#include <espace/input.h>
#include <espace/image.h>
#include <espace/map.h>
#include <espace/media.h>
#include <espace/montage.h>
#include <espace/opengl.h>
#include <espace/plugins.h>
#include <espace/preprocessor.h>
#include <espace/renderer.h>
#include <espace/shader.h>
#include <espace/sound.h>
#include <espace/system.h>
%}

typedef unsigned int uint;
typedef const char* String;

#define IMPORT

%nodefault Console;
%nodefault Input;
%nodefault Media;
%nodefault Renderer;
%nodefault Shader;
%nodefault Skin;

%include ../include/espace/collision.h
%include ../include/espace/color.h
%include ../include/espace/console.h
%include ../include/espace/cvar.h
%include ../include/espace/input.h
%include ../include/espace/media.h
%include ../include/espace/montage.h
%include ../include/espace/shader.h
%include ../include/espace/skin.h
%include ../include/espace/sound.h
%include ../include/espace/map.h
%include ../include/espace/matrix.h
%include ../include/espace/model.h
%include ../include/espace/preprocessor.h
%include ../include/espace/system.h
%include ../include/espace/renderer.h
%include ../include/espace/vector.h

%{
  static void pythonCommand()
  {
    PyRun_SimpleString(API::args());
  }

  static void pythonPlusCommand()
  {
    PyRun_SimpleString(String("plus") + API::args());
  }

  static void pythonMinusCommand()
  {
    PyRun_SimpleString(String("minus") + API::args());
  }
%}

%nodefault API;
struct API
{
  typedef void (*Command)(void);

  static void initialize();

  static void setCommand(const char*, Command);

  static void addCommand(const char*);
  static void flushCommands();
  static void executeCommand(const char*);
  static void executeClientCommand(const char*, int clientNum);

  static uint argc();
  static const char* argv(int index);
};

%extend API
{
  static void pythonInitialize()
  {
    API::setCommand("eval", pythonCommand);
    API::setCommand("+eval", pythonPlusCommand);
    API::setCommand("-eval", pythonMinusCommand);
  }
};

%extend Entity
{
  const char* get(const char* key)
  {
    if(self->find(key) == self->end())
      return 0;

    return (*self)[key];
  }

  void set(const char* key, const char* value)
  {
    (*self)[key] = value;
  }
};

struct File
{
  static bool readDirectory(const char* directory, const char* prefix = "");

  File(const char* fileName, unsigned int bufferSize = 65536);

  static const char* realName(const char* name);
};

%extend Shader
{
  static void setTime(float time)
  {
    Shader::time = time;
  }
}

struct Image
{
  static Image* acquireSimple(const char* name);
};

%nodefault Plugin;
struct Plugin
{
  static void initialize();
};

struct GLConfig
{
  enum DriverType
  {
    GLDRV_ICD,
    GLDRV_STANDALONE,
    GLDRV_VOODOO
  };

  enum HardwareType
  {
    GLHW_GENERIC,
    GLHW_3DFX_2D3D,
    GLHW_RIVA128,
    GLHW_RAGEPRO,
    GLHW_PERMEDIA2
  };

  enum TextureCompression
  {
    TC_NONE,
    TC_S3TC,
    TC_EXT_COMP_S3TC
  };

  char  renderer[1024];
  char  vendor[1024];
  char  version[1024];
  char  extensions[4096];

  uint  maxTextureSize;
  uint  maxActiveTextures;

  uint  colorBits;
  uint  depthBits;
  uint  stencilBits;

  DriverType  driverType;
  HardwareType hardwareType;

  int   gammaSupport;
  TextureCompression textureCompression;
  int   textureEnvAddSupport;
  int   anisotropicSupport;
  float maxAnisotropy;

  int   NVFogAvailable;
  int   NVFogMode;

  int   ATIMaxTruformTess;
  int   ATINormalMode;
  int   ATIPointMode;

  int   width;
  int   height;
  float aspect;

  int   refreshRate;

  int   fullscreen;
  int   stereo;
  int   smp;

  int   textureFilterAnisotropicSupport;
};

%nodefault GL;
struct GL
{
  static void initialize();

  static void clearIndex(float c);
  static void clearColor(float red, float green, float blue,
                               float alpha);
  static void clear(unsigned int mask);
  static void indexMask(unsigned int mask);
  static void colorMask(unsigned char red, unsigned char green, unsigned char blue,
                              unsigned char alpha);
  static void alphaFunc(unsigned int func, float ref);
  static void blendFunc(unsigned int sfactor, unsigned int dfactor);
  static void logicOp(unsigned int opcode);
  static void cullFace(unsigned int mode);
  static void frontFace(unsigned int mode);
  static void pointSize(float size);
  static void lineWidth(float width);
  static void lineStipple(int factor, unsigned short pattern);
  static void polygonMode(unsigned int face, unsigned int mode);
  static void polygonOffset(float factor, float units);
  static void polygonStipple(const unsigned char *mask);
  static void getPolygonStipple(unsigned char *mask);
  static void edgeFlag(unsigned char flag);
  static void edgeFlagv(const unsigned char *flag);
  static void scissor(int x, int y, int width, int height);
  static void clipPlane(unsigned int plane, const double *equation);
  static void getClipPlane(unsigned int plane, double *equation);
  static void drawBuffer(unsigned int mode);
  static void readBuffer(unsigned int mode);
  static void enable(unsigned int cap);
  static void disable(unsigned int cap);
  static unsigned char isEnabled(unsigned int cap);
  static void enableClientState(unsigned int cap);
  static void disableClientState(unsigned int cap);
  static void getBooleanv(unsigned int pname, unsigned char *params);
  static void getDoublev(unsigned int pname, double *params);
  static void getFloatv(unsigned int pname, float *params);
  static void getIntegerv(unsigned int pname, int *params);
  static void pushAttrib(unsigned int mask);
  static void popAttrib(void);
  static void pushClientAttrib(unsigned int mask);
  static void popClientAttrib(void);
  static int renderMode(unsigned int mode);
  static unsigned int getError(void);
  static const unsigned char* getString(unsigned int name);
  static void finish(void);
  static void flush(void);
  static void hint(unsigned int target, unsigned int mode);
  static void clearDepth(double depth);
  static void depthFunc(unsigned int func);
  static void depthMask(unsigned char flag);
  static void depthRange(double near_val, double far_val);
  static void clearAccum(float red, float green, float blue,
                               float alpha);
  static void accum(unsigned int op, float value);
  static void matrixMode(unsigned int mode);
  static void ortho(double left, double right, double bottom,
                          double top, double near_val, double far_val);
  static void frustum(double left, double right, double bottom,
                            double top, double near_val, double far_val);
  static void viewport(int x, int y, int width, int height);
  static void pushMatrix(void);
  static void popMatrix(void);
  static void loadIdentity(void);
  static void loadMatrixd(const double *m);
  static void loadMatrixf(const float *m);
  static void multMatrixd(const double *m);
  static void multMatrixf(const float *m);
  static void rotated(double angle, double x, double y, double z);
  static void rotatef(float angle, float x, float y, float z);
  static void scaled(double x, double y, double z);
  static void scalef(float x, float y, float z);
  static void translated(double x, double y, double z);
  static void translatef(float x, float y, float z);
  static unsigned char isList(unsigned int list);
  static void deleteLists(unsigned int list, int range);
  static unsigned int genLists(int range);
  static void newList(unsigned int list, unsigned int mode);
  static void endList(void);
  static void callList(unsigned int list);
  static void callLists(int n, unsigned int type, const void *lists);
  static void listBase(unsigned int base);
  static void begin(unsigned int mode);
  static void end(void);
  static void vertex2d(double x, double y);
  static void vertex2f(float x, float y);
  static void vertex2i(int x, int y);
  static void vertex2s(short x, short y);
  static void vertex3d(double x, double y, double z);
  static void vertex3f(float x, float y, float z);
  static void vertex3i(int x, int y, int z);
  static void vertex3s(short x, short y, short z);
  static void vertex4d(double x, double y, double z, double w);
  static void vertex4f(float x, float y, float z, float w);
  static void vertex4i(int x, int y, int z, int w);
  static void vertex4s(short x, short y, short z, short w);
  static void vertex2dv(const double *v);
  static void vertex2fv(const float *v);
  static void vertex2iv(const int *v);
  static void vertex2sv(const short *v);
  static void vertex3dv(const double *v);
  static void vertex3fv(const float *v);
  static void vertex3iv(const int *v);
  static void vertex3sv(const short *v);
  static void vertex4dv(const double *v);
  static void vertex4fv(const float *v);
  static void vertex4iv(const int *v);
  static void vertex4sv(const short *v);
  static void normal3b(signed char nx, signed char ny, signed char nz);
  static void normal3d(double nx, double ny, double nz);
  static void normal3f(float nx, float ny, float nz);
  static void normal3i(int nx, int ny, int nz);
  static void normal3s(short nx, short ny, short nz);
  static void normal3bv(const signed char *v);
  static void normal3dv(const double *v);
  static void normal3fv(const float *v);
  static void normal3iv(const int *v);
  static void normal3sv(const short *v);
  static void indexd(double c);
  static void indexf(float c);
  static void indexi(int c);
  static void indexs(short c);
  static void indexub(unsigned char c);
  static void indexdv(const double *c);
  static void indexfv(const float *c);
  static void indexiv(const int *c);
  static void indexsv(const short *c);
  static void indexubv(const unsigned char *c);
  static void color3b(signed char red, signed char green, signed char blue);
  static void color3d(double red, double green, double blue);
  static void color3f(float red, float green, float blue);
  static void color3i(int red, int green, int blue);
  static void color3s(short red, short green, short blue);
  static void color3ub(unsigned char red, unsigned char green, unsigned char blue);
  static void color3ui(unsigned int red, unsigned int green, unsigned int blue);
  static void color3us(unsigned short red, unsigned short green, unsigned short blue);
  static void color4b(signed char red, signed char green, signed char blue,
                            signed char alpha);
  static void color4d(double red, double green, double blue,
                            double alpha);
  static void color4f(float red, float green, float blue,
                            float alpha);
  static void color4i(int red, int green, int blue, int alpha);
  static void color4s(short red, short green, short blue,
                            short alpha);
  static void color4ub(unsigned char red, unsigned char green, unsigned char blue,
                             unsigned char alpha);
  static void color4ui(unsigned int red, unsigned int green, unsigned int blue,
                             unsigned int alpha);
  static void color4us(unsigned short red, unsigned short green, unsigned short blue,
                             unsigned short alpha);
  static void color3bv(const signed char *v);
  static void color3dv(const double *v);
  static void color3fv(const float *v);
  static void color3iv(const int *v);
  static void color3sv(const short *v);
  static void color3ubv(const unsigned char *v);
  static void color3uiv(const unsigned int *v);
  static void color3usv(const unsigned short *v);
  static void color4bv(const signed char *v);
  static void color4dv(const double *v);
  static void color4fv(const float *v);
  static void color4iv(const int *v);
  static void color4sv(const short *v);
  static void color4ubv(const unsigned char *v);
  static void color4uiv(const unsigned int *v);
  static void color4usv(const unsigned short *v);
  static void texCoord1d(double s);
  static void texCoord1f(float s);
  static void texCoord1i(int s);
  static void texCoord1s(short s);
  static void texCoord2d(double s, double t);
  static void texCoord2f(float s, float t);
  static void texCoord2i(int s, int t);
  static void texCoord2s(short s, short t);
  static void texCoord3d(double s, double t, double r);
  static void texCoord3f(float s, float t, float r);
  static void texCoord3i(int s, int t, int r);
  static void texCoord3s(short s, short t, short r);
  static void texCoord4d(double s, double t, double r, double q);
  static void texCoord4f(float s, float t, float r, float q);
  static void texCoord4i(int s, int t, int r, int q);
  static void texCoord4s(short s, short t, short r, short q);
  static void texCoord1dv(const double *v);
  static void texCoord1fv(const float *v);
  static void texCoord1iv(const int *v);
  static void texCoord1sv(const short *v);
  static void texCoord2dv(const double *v);
  static void texCoord2fv(const float *v);
  static void texCoord2iv(const int *v);
  static void texCoord2sv(const short *v);
  static void texCoord3dv(const double *v);
  static void texCoord3fv(const float *v);
  static void texCoord3iv(const int *v);
  static void texCoord3sv(const short *v);
  static void texCoord4dv(const double *v);
  static void texCoord4fv(const float *v);
  static void texCoord4iv(const int *v);
  static void texCoord4sv(const short *v);
  static void rasterPos2d(double x, double y);
  static void rasterPos2f(float x, float y);
  static void rasterPos2i(int x, int y);
  static void rasterPos2s(short x, short y);
  static void rasterPos3d(double x, double y, double z);
  static void rasterPos3f(float x, float y, float z);
  static void rasterPos3i(int x, int y, int z);
  static void rasterPos3s(short x, short y, short z);
  static void rasterPos4d(double x, double y, double z, double w);
  static void rasterPos4f(float x, float y, float z, float w);
  static void rasterPos4i(int x, int y, int z, int w);
  static void rasterPos4s(short x, short y, short z, short w);
  static void rasterPos2dv(const double *v);
  static void rasterPos2fv(const float *v);
  static void rasterPos2iv(const int *v);
  static void rasterPos2sv(const short *v);
  static void rasterPos3dv(const double *v);
  static void rasterPos3fv(const float *v);
  static void rasterPos3iv(const int *v);
  static void rasterPos3sv(const short *v);
  static void rasterPos4dv(const double *v);
  static void rasterPos4fv(const float *v);
  static void rasterPos4iv(const int *v);
  static void rasterPos4sv(const short *v);
  static void rectd(double x1, double y1, double x2, double y2);
  static void rectf(float x1, float y1, float x2, float y2);
  static void recti(int x1, int y1, int x2, int y2);
  static void rects(short x1, short y1, short x2, short y2);
  static void rectdv(const double *v1, const double *v2);
  static void rectfv(const float *v1, const float *v2);
  static void rectiv(const int *v1, const int *v2);
  static void rectsv(const short *v1, const short *v2);
  static void vertexPointer(int size, unsigned int type, int stride,
                                  const void *ptr);
  static void normalPointer(unsigned int type, int stride,
                                  const void *ptr);
  static void colorPointer(int size, unsigned int type, int stride,
                                 const void *ptr);
  static void indexPointer(unsigned int type, int stride,
                                 const void *ptr);
  static void texCoordPointer(int size, unsigned int type, int stride,
                                    const void *ptr);
  static void edgeFlagPointer(int stride, const void *ptr);
  static void getPointerv(unsigned int pname, void **params);
  static void arrayElement(int i);
  static void drawArrays(unsigned int mode, int first, int count);
  static void drawElements(unsigned int mode, int count, unsigned int type,
                                 const void *indices);
  static void interleavedArrays(unsigned int format, int stride,
                                      const void *pointer);
  static void shadeModel(unsigned int mode);
  static void lightf(unsigned int light, unsigned int pname, float param);
  static void lighti(unsigned int light, unsigned int pname, int param);
  static void lightfv(unsigned int light, unsigned int pname, const float *params);
  static void lightiv(unsigned int light, unsigned int pname, const int *params);
  static void getLightfv(unsigned int light, unsigned int pname, float *params);
  static void getLightiv(unsigned int light, unsigned int pname, int *params);
  static void lightModelf(unsigned int pname, float param);
  static void lightModeli(unsigned int pname, int param);
  static void lightModelfv(unsigned int pname, const float *params);
  static void lightModeliv(unsigned int pname, const int *params);
  static void materialf(unsigned int face, unsigned int pname, float param);
  static void materiali(unsigned int face, unsigned int pname, int param);
  static void materialfv(unsigned int face, unsigned int pname,
                               const float *params);
  static void materialiv(unsigned int face, unsigned int pname, const int *params);
  static void getMaterialfv(unsigned int face, unsigned int pname, float *params);
  static void getMaterialiv(unsigned int face, unsigned int pname, int *params);
  static void colorMaterial(unsigned int face, unsigned int mode);
  static void pixelZoom(float xfactor, float yfactor);
  static void pixelStoref(unsigned int pname, float param);
  static void pixelStorei(unsigned int pname, int param);
  static void pixelTransferf(unsigned int pname, float param);
  static void pixelTransferi(unsigned int pname, int param);
  static void pixelMapfv(unsigned int map, int mapsize,
                               const float *values);
  static void pixelMapuiv(unsigned int map, int mapsize,
                                const unsigned int *values);
  static void pixelMapusv(unsigned int map, int mapsize,
                                const unsigned short *values);
  static void getPixelMapfv(unsigned int map, float *values);
  static void getPixelMapuiv(unsigned int map, unsigned int *values);
  static void getPixelMapusv(unsigned int map, unsigned short *values);
  static void bitmap(int width, int height, float xorig,
                           float yorig, float xmove, float ymove,
                           const unsigned char *bitmap);
  static void readPixels(int x, int y, int width, int height,
                               unsigned int format, unsigned int type, void *pixels);
  static void drawPixels(int width, int height, unsigned int format,
                               unsigned int type, const void *pixels);
  static void copyPixels(int x, int y, int width, int height,
                               unsigned int type);
  static void stencilFunc(unsigned int func, int ref, unsigned int mask);
  static void stencilMask(unsigned int mask);
  static void stencilOp(unsigned int fail, unsigned int zfail, unsigned int zpass);
  static void clearStencil(int s);
  static void texGend(unsigned int coord, unsigned int pname, double param);
  static void texGenf(unsigned int coord, unsigned int pname, float param);
  static void texGeni(unsigned int coord, unsigned int pname, int param);
  static void texGendv(unsigned int coord, unsigned int pname,
                             const double *params);
  static void texGenfv(unsigned int coord, unsigned int pname, const float *params);
  static void texGeniv(unsigned int coord, unsigned int pname, const int *params);
  static void getTexGendv(unsigned int coord, unsigned int pname, double *params);
  static void getTexGenfv(unsigned int coord, unsigned int pname, float *params);
  static void getTexGeniv(unsigned int coord, unsigned int pname, int *params);
  static void texEnvf(unsigned int target, unsigned int pname, float param);
  static void texEnvi(unsigned int target, unsigned int pname, int param);
  static void texEnvfv(unsigned int target, unsigned int pname,
                             const float *params);
  static void texEnviv(unsigned int target, unsigned int pname, const int *params);
  static void getTexEnvfv(unsigned int target, unsigned int pname, float *params);
  static void getTexEnviv(unsigned int target, unsigned int pname, int *params);
  static void texParameterf(unsigned int target, unsigned int pname, float param);
  static void texParameteri(unsigned int target, unsigned int pname, int param);
  static void texParameterfv(unsigned int target, unsigned int pname,
                                   const float *params);
  static void texParameteriv(unsigned int target, unsigned int pname,
                                   const int *params);
  static void getTexParameterfv(unsigned int target, unsigned int pname,
                                      float *params);
  static void getTexParameteriv(unsigned int target, unsigned int pname, 
                                      int *params);
  static void getTexLevelParameterfv(unsigned int target, int level,
                                           unsigned int pname, float *params);
  static void getTexLevelParameteriv(unsigned int target, int level,
                                           unsigned int pname, int *params);
  static void texImage1D(unsigned int target, int level, int internalFormat,
                               int width, int border, unsigned int format,
                               unsigned int type, const void *pixels);
  static void texImage2D(unsigned int target, int level, int internalFormat,
                               int width, int height, int border,
                               unsigned int format, unsigned int type,
                               const void *pixels);
  static void getTexImage(unsigned int target, int level, unsigned int format,
                                unsigned int type, void *pixels);
  static void genTextures(int n, unsigned int *textures);
  static void deleteTextures(int n, const unsigned int *textures);
  static void bindTexture(unsigned int target, unsigned int texture);
  static void prioritizeTextures(int n, const unsigned int *textures,
                                       const float *priorities);
  static unsigned char areTexturesResident(int n, const unsigned int *textures,
                                             unsigned char *residences);
  static unsigned char isTexture(unsigned int texture);
  static void texSubImage1D(unsigned int target, int level, int xoffset,
                                  int width, unsigned int format, unsigned int type,
                                  const void *pixels);
  static void texSubImage2D(unsigned int target, int level, int xoffset,
                                  int yoffset, int width, int height,
                                  unsigned int format, unsigned int type,
                                  const void *pixels);
  static void copyTexImage1D(unsigned int target, int level,
                                   unsigned int internalformat, int x, int y,
                                   int width, int border);
  static void copyTexImage2D(unsigned int target, int level,
                                   unsigned int internalformat, int x, int y,
                                   int width, int height, int border);
  static void copyTexSubImage1D(unsigned int target, int level, int xoffset,
                                      int x, int y, int width);
  static void copyTexSubImage2D(unsigned int target, int level, int xoffset,
                                      int yoffset, int x, int y,
                                      int width, int height);
  static void map1d(unsigned int target, double u1, double u2, int stride,
                          int order, const double *points);
  static void map1f(unsigned int target, float u1, float u2, int stride, int order, const float *points);
  static void map2d(unsigned int target, double u1, double u2,
                          int ustride, int uorder, double v1, double v2,
                          int vstride, int vorder, const double *points);
  static void map2f(unsigned int target, float u1, float u2, int ustride,
                          int uorder, float v1, float v2, int vstride,
                          int vorder, const float *points);
  static void getMapdv(unsigned int target, unsigned int query, double *v);
  static void getMapfv(unsigned int target, unsigned int query, float *v);
  static void getMapiv(unsigned int target, unsigned int query, int *v);
  static void evalCoord1d(double u);
  static void evalCoord1f(float u);
  static void evalCoord1dv(const double *u);
  static void evalCoord1fv(const float *u);
  static void evalCoord2d(double u, double v);
  static void evalCoord2f(float u, float v);
  static void evalCoord2dv(const double *u);
  static void evalCoord2fv(const float *u);
  static void mapGrid1d(int un, double u1, double u2);
  static void mapGrid1f(int un, float u1, float u2);
  static void mapGrid2d(int un, double u1, double u2, int vn,
                              double v1, double v2);
  static void mapGrid2f(int un, float u1, float u2, int vn,
                              float v1, float v2);
  static void evalPoint1(int i);
  static void evalPoint2(int i, int j);
  static void evalMesh1(unsigned int mode, int i1, int i2);
  static void evalMesh2(unsigned int mode, int i1, int i2, int j1,
                              int j2);
  static void fogf(unsigned int pname, float param);
  static void fogi(unsigned int pname, int param);
  static void fogfv(unsigned int pname, const float *params);
  static void fogiv(unsigned int pname, const int *params);
  static void feedbackBuffer(int size, unsigned int type, float *buffer);
  static void passThrough(float token);
  static void selectBuffer(int size, unsigned int *buffer);
  static void initNames(void);
  static void loadName(unsigned int name);
  static void pushName(unsigned int name);
  static void popName(void);
  static void drawRangeElements(unsigned int mode, unsigned int start, unsigned int end,
                                      int count, unsigned int type,
                                      const void *indices);
  static void texImage3D(unsigned int target, int level, int internalFormat,
                               int width, int height, int depth,
                               int border, unsigned int format, unsigned int type,
                               const void *pixels);
  static void texSubImage3D(unsigned int target, int level, int xoffset,
                                  int yoffset, int zoffset, int width,
                                  int height, int depth, unsigned int format,
                                  unsigned int type, const void *pixels);
  static void copyTexSubImage3D(unsigned int target, int level, int xoffset,
                                      int yoffset, int zoffset, int x,
                                      int y, int width, int height);
  static void colorTable(unsigned int target, unsigned int internalformat,
                               int width, unsigned int format, unsigned int type,
                               const void *table);
  static void colorSubTable(unsigned int target, int start, int count,
                                  unsigned int format, unsigned int type,
                                 const void *data);
  static void colorTableParameteriv(unsigned int target, unsigned int pname,
                                          const int *params);
  static void colorTableParameterfv(unsigned int target, unsigned int pname,
                                          const float *params);
  static void copyColorSubTable(unsigned int target, int start, int x,
                                      int y, int width);
  static void copyColorTable(unsigned int target, unsigned int internalformat,
                                   int x, int y, int width);
  static void getColorTable(unsigned int target, unsigned int format, unsigned int type,
                                  void *table);
  static void getColorTableParameterfv(unsigned int target, unsigned int pname,
                                             float *params);
  static void getColorTableParameteriv(unsigned int target, unsigned int pname,
                                             int *params);
  static void blendEquation(unsigned int mode);
  static void blendColor(float red, float green, float blue,
                               float alpha);
  static void histogram(unsigned int target, int width,
                              unsigned int internalformat, unsigned char sink);
  static void resetHistogram(unsigned int target);
  static void getHistogram(unsigned int target, unsigned char reset, unsigned int format,
                                 unsigned int type, void *values);
  static void getHistogramParameterfv(unsigned int target, unsigned int pname,
                                            float *params);
  static void getHistogramParameteriv(unsigned int target, unsigned int pname,
                                            int *params);
  static void minmax(unsigned int target, unsigned int internalformat,
                           unsigned char sink);
  static void resetMinmax(unsigned int target);
  static void getMinmax(unsigned int target, unsigned char reset, unsigned int format,
                              unsigned int types, void *values);
  static void getMinmaxParameterfv(unsigned int target, unsigned int pname,
                                         float *params);
  static void getMinmaxParameteriv(unsigned int target, unsigned int pname,
                                         int *params);
  static void convolutionFilter1D(unsigned int target, unsigned int internalformat,
                                        int width, unsigned int format,
                                        unsigned int type, const void *image);
  static void convolutionFilter2D(unsigned int target, unsigned int internalformat,
                                        int width, int height,
                                        unsigned int format, unsigned int type,
                                        const void *image);
  static void convolutionParameterf(unsigned int target, unsigned int pname,
                                          float params);
  static void convolutionParameterfv(unsigned int target, unsigned int pname,
                                           const float *params);
  static void convolutionParameteri(unsigned int target, unsigned int pname,
                                          int params);
  static void convolutionParameteriv(unsigned int target, unsigned int pname,
                                           const int *params);
  static void copyConvolutionFilter1D(unsigned int target,
                                            unsigned int internalformat,
                                            int x, int y, int width);
  static void copyConvolutionFilter2D(unsigned int target,
                                            unsigned int internalformat, int x,
                                            int y, int width,
                                            int height);
  static void getConvolutionFilter(unsigned int target, unsigned int format,
                                         unsigned int type, void *image);
  static void getConvolutionParameterfv(unsigned int target, unsigned int pname,
                                              float *params);
  static void getConvolutionParameteriv(unsigned int target, unsigned int pname,
                                              int *params);
  static void separableFilter2D(unsigned int target, unsigned int internalformat,
                                      int width, int height,
                                      unsigned int format, unsigned int type,
                                      const void *row, const void *column);
  static void getSeparableFilter(unsigned int target, unsigned int format,
                                       unsigned int type, void *row, void *column,
                                       void *span);
  static void activeTextureARB(unsigned int texture);
  static void clientActiveTextureARB(unsigned int texture);
  static void multiTexCoord1dARB(unsigned int target, double s);
  static void multiTexCoord1dvARB(unsigned int target, const double *v);
  static void multiTexCoord1fARB(unsigned int target, float s);
  static void multiTexCoord1fvARB(unsigned int target, const float *v);
  static void multiTexCoord1iARB(unsigned int target, int s);
  static void multiTexCoord1ivARB(unsigned int target, const int *v);
  static void multiTexCoord1sARB(unsigned int target, short s);
  static void multiTexCoord1svARB(unsigned int target, const short *v);
  static void multiTexCoord2dARB(unsigned int target, double s, double t);
  static void multiTexCoord2dvARB(unsigned int target, const double *v);
  static void multiTexCoord2fARB(unsigned int target, float s, float t);
  static void multiTexCoord2fvARB(unsigned int target, const float *v);
  static void multiTexCoord2iARB(unsigned int target, int s, int t);
  static void multiTexCoord2ivARB(unsigned int target, const int *v);
  static void multiTexCoord2sARB(unsigned int target, short s, short t);
  static void multiTexCoord2svARB(unsigned int target, const short *v);
  static void multiTexCoord3dARB(unsigned int target, double s, double t,
                                       double r);
  static void multiTexCoord3dvARB(unsigned int target, const double *v);
  static void multiTexCoord3fARB(unsigned int target, float s, float t,
                                       float r);
  static void multiTexCoord3fvARB(unsigned int target, const float *v);
  static void multiTexCoord3iARB(unsigned int target, int s, int t,
                                       int r);
  static void multiTexCoord3ivARB(unsigned int target, const int *v);
  static void multiTexCoord3sARB(unsigned int target, short s, short t,
                                       short r);
  static void multiTexCoord3svARB(unsigned int target, const short *v);
  static void multiTexCoord4dARB(unsigned int target, double s, double t,
                                       double r, double q);
  static void multiTexCoord4dvARB(unsigned int target, const double *v);
  static void multiTexCoord4fARB(unsigned int target, float s, float t,
                                       float r, float q);
  static void multiTexCoord4fvARB(unsigned int target, const float *v);
  static void multiTexCoord4iARB(unsigned int target, int s, int t, int r,
                                       int q);
  static void multiTexCoord4ivARB(unsigned int target, const int *v);
  static void multiTexCoord4sARB(unsigned int target, short s, short t,
                                       short r, short q);
  static void multiTexCoord4svARB(unsigned int target, const short *v);
  static void blendColorEXT(float red, float green, float blue,
                                  float alpha);
  static void polygonOffsetEXT(float factor, float bias);
  static void texImage3DEXT(unsigned int target, int level,
                                  unsigned int internalFormat, int width,
                                  int height, int depth, int border,
                                  unsigned int format, unsigned int type,
                                  const void *pixels);
  static void texSubImage3DEXT(unsigned int target, int level, int xoffset,
                                     int yoffset, int zoffset,
                                     int width, int height,
                                     int depth, unsigned int format, unsigned int type,
                                     const void *pixels);
  static void copyTexSubImage3DEXT(unsigned int target, int level,
                                         int xoffset, int yoffset,
                                         int zoffset, int x, int y,
                                         int width, int height);
  static void genTexturesEXT(int n, unsigned int *textures);
  static void deleteTexturesEXT(int n, const unsigned int *textures);
  static void bindTextureEXT(unsigned int target, unsigned int texture);
  static void prioritizeTexturesEXT(int n, const unsigned int *textures,
                                          const float *priorities);
  static unsigned char areTexturesResidentEXT(int n,
                                                const unsigned int *textures,
                                                unsigned char *residences);
  static unsigned char isTextureEXT(unsigned int texture);
  static void vertexPointerEXT(int size, unsigned int type, int stride,
                                     int count, const void *ptr);
  static void normalPointerEXT(unsigned int type, int stride, int count,
                                     const void *ptr);
  static void colorPointerEXT(int size, unsigned int type, int stride,
                                    int count, const void *ptr);
  static void indexPointerEXT(unsigned int type, int stride, int count,
                                    const void *ptr);
  static void texCoordPointerEXT(int size, unsigned int type, int stride,
                                       int count, const void *ptr);
  static void edgeFlagPointerEXT(int stride, int count,
                                       const unsigned char *ptr);
  static void getPointervEXT(unsigned int pname, void **params);
  static void arrayElementEXT(int i);
  static void drawArraysEXT(unsigned int mode, int first, int count);
  static void blendEquationEXT(unsigned int mode);
  static void pointParameterfEXT(unsigned int pname, float param);
  static void pointParameterfvEXT(unsigned int pname, const float *params);
  static void pointParameterfSGIS(unsigned int pname, float param);
  static void pointParameterfvSGIS(unsigned int pname, const float *params);
  static void colorTableEXT(unsigned int target, unsigned int internalformat,
                                  int width, unsigned int format, unsigned int type,
                                  const void *table);
  static void colorSubTableEXT(unsigned int target, int start,
                                     int count, unsigned int format, unsigned int type,
                                     const void *data);
  static void getColorTableEXT(unsigned int target, unsigned int format, unsigned int type,
                                     void *table);
  static void getColorTableParameterfvEXT(unsigned int target, unsigned int pname,
                                                float *params);
  static void getColorTableParameterivEXT(unsigned int target, unsigned int pname,
                                                int *params);
  static void lockArraysEXT(int first, int count);
  static void unlockArraysEXT(void);
  static void windowPos2iMESA(int x, int y);
  static void windowPos2sMESA(short x, short y);
  static void windowPos2fMESA(float x, float y);
  static void windowPos2dMESA(double x, double y);
  static void windowPos2ivMESA(const int *p);
  static void windowPos2svMESA(const short *p);
  static void windowPos2fvMESA(const float *p);
  static void windowPos2dvMESA(const double *p);
  static void windowPos3iMESA(int x, int y, int z);
  static void windowPos3sMESA(short x, short y, short z);
  static void windowPos3fMESA(float x, float y, float z);
  static void windowPos3dMESA(double x, double y, double z);
  static void windowPos3ivMESA(const int *p);
  static void windowPos3svMESA(const short *p);
  static void windowPos3fvMESA(const float *p);
  static void windowPos3dvMESA(const double *p);
  static void windowPos4iMESA(int x, int y, int z, int w);
  static void windowPos4sMESA(short x, short y, short z, short w);
  static void windowPos4fMESA(float x, float y, float z, float w);
  static void windowPos4dMESA(double x, double y, double z,
                                    double w);
  static void windowPos4ivMESA(const int *p);
  static void windowPos4svMESA(const short *p);
  static void windowPos4fvMESA(const float *p);
  static void windowPos4dvMESA(const double *p);
  static void resizeBuffersMESA(void);

  enum
  {
    VERSION_1_1 = 1,
    VERSION_1_2 = 1,
    FALSE = 0x0,
    TRUE = 0x1,
    BYTE = 0x1400,
    UNSIGNED_BYTE = 0x1401,
    SHORT = 0x1402,
    UNSIGNED_SHORT = 0x1403,
    INT = 0x1404,
    UNSIGNED_INT = 0x1405,
    FLOAT = 0x1406,
    DOUBLE = 0x140A,
    // 2_BYTES = 0x1407,
    // 3_BYTES = 0x1408,
    // 4_BYTES = 0x1409,
    POINTS = 0x0000,
    LINES = 0x0001,
    LINE_LOOP = 0x0002,
    LINE_STRIP = 0x0003,
    TRIANGLES = 0x0004,
    TRIANGLE_STRIP = 0x0005,
    TRIANGLE_FAN = 0x0006,
    QUADS = 0x0007,
    QUAD_STRIP = 0x0008,
    POLYGON = 0x0009,
    VERTEX_ARRAY = 0x8074,
    NORMAL_ARRAY = 0x8075,
    COLOR_ARRAY = 0x8076,
    INDEX_ARRAY = 0x8077,
    TEXTURE_COORD_ARRAY = 0x8078,
    EDGE_FLAG_ARRAY = 0x8079,
    VERTEX_ARRAY_SIZE = 0x807A,
    VERTEX_ARRAY_TYPE = 0x807B,
    VERTEX_ARRAY_STRIDE = 0x807C,
    NORMAL_ARRAY_TYPE = 0x807E,
    NORMAL_ARRAY_STRIDE = 0x807F,
    COLOR_ARRAY_SIZE = 0x8081,
    COLOR_ARRAY_TYPE = 0x8082,
    COLOR_ARRAY_STRIDE = 0x8083,
    INDEX_ARRAY_TYPE = 0x8085,
    INDEX_ARRAY_STRIDE = 0x8086,
    TEXTURE_COORD_ARRAY_SIZE = 0x8088,
    TEXTURE_COORD_ARRAY_TYPE = 0x8089,
    TEXTURE_COORD_ARRAY_STRIDE = 0x808A,
    EDGE_FLAG_ARRAY_STRIDE = 0x808C,
    VERTEX_ARRAY_POINTER = 0x808E,
    NORMAL_ARRAY_POINTER = 0x808F,
    COLOR_ARRAY_POINTER = 0x8090,
    INDEX_ARRAY_POINTER = 0x8091,
    TEXTURE_COORD_ARRAY_POINTER = 0x8092,
    EDGE_FLAG_ARRAY_POINTER = 0x8093,
    V2F = 0x2A20,
    V3F = 0x2A21,
    C4UB_V2F = 0x2A22,
    C4UB_V3F = 0x2A23,
    C3F_V3F = 0x2A24,
    N3F_V3F = 0x2A25,
    C4F_N3F_V3F = 0x2A26,
    T2F_V3F = 0x2A27,
    T4F_V4F = 0x2A28,
    T2F_C4UB_V3F = 0x2A29,
    T2F_C3F_V3F = 0x2A2A,
    T2F_N3F_V3F = 0x2A2B,
    T2F_C4F_N3F_V3F = 0x2A2C,
    T4F_C4F_N3F_V4F = 0x2A2D,
    MATRIX_MODE = 0x0BA0,
    MODELVIEW = 0x1700,
    PROJECTION = 0x1701,
    TEXTURE = 0x1702,
    POINT_SMOOTH = 0x0B10,
    POINT_SIZE = 0x0B11,
    POINT_SIZE_GRANULARITY = 0x0B13,
    POINT_SIZE_RANGE = 0x0B12,
    LINE_SMOOTH = 0x0B20,
    LINE_STIPPLE = 0x0B24,
    LINE_STIPPLE_PATTERN = 0x0B25,
    LINE_STIPPLE_REPEAT = 0x0B26,
    LINE_WIDTH = 0x0B21,
    LINE_WIDTH_GRANULARITY = 0x0B23,
    LINE_WIDTH_RANGE = 0x0B22,
    POINT = 0x1B00,
    LINE = 0x1B01,
    FILL = 0x1B02,
    CW = 0x0900,
    CCW = 0x0901,
    FRONT = 0x0404,
    BACK = 0x0405,
    POLYGON_MODE = 0x0B40,
    POLYGON_SMOOTH = 0x0B41,
    POLYGON_STIPPLE = 0x0B42,
    EDGE_FLAG = 0x0B43,
    CULL_FACE = 0x0B44,
    CULL_FACE_MODE = 0x0B45,
    FRONT_FACE = 0x0B46,
    POLYGON_OFFSET_FACTOR = 0x8038,
    POLYGON_OFFSET_UNITS = 0x2A00,
    POLYGON_OFFSET_POINT = 0x2A01,
    POLYGON_OFFSET_LINE = 0x2A02,
    POLYGON_OFFSET_FILL = 0x8037,
    COMPILE = 0x1300,
    COMPILE_AND_EXECUTE = 0x1301,
    LIST_BASE = 0x0B32,
    LIST_INDEX = 0x0B33,
    LIST_MODE = 0x0B30,
    NEVER = 0x0200,
    LESS = 0x0201,
    EQUAL = 0x0202,
    LEQUAL = 0x0203,
    GREATER = 0x0204,
    NOTEQUAL = 0x0205,
    GEQUAL = 0x0206,
    ALWAYS = 0x0207,
    DEPTH_TEST = 0x0B71,
    DEPTH_BITS = 0x0D56,
    DEPTH_CLEAR_VALUE = 0x0B73,
    DEPTH_FUNC = 0x0B74,
    DEPTH_RANGE = 0x0B70,
    DEPTH_WRITEMASK = 0x0B72,
    DEPTH_COMPONENT = 0x1902,
    LIGHTING = 0x0B50,
    LIGHT0 = 0x4000,
    LIGHT1 = 0x4001,
    LIGHT2 = 0x4002,
    LIGHT3 = 0x4003,
    LIGHT4 = 0x4004,
    LIGHT5 = 0x4005,
    LIGHT6 = 0x4006,
    LIGHT7 = 0x4007,
    SPOT_EXPONENT = 0x1205,
    SPOT_CUTOFF = 0x1206,
    CONSTANT_ATTENUATION = 0x1207,
    LINEAR_ATTENUATION = 0x1208,
    QUADRATIC_ATTENUATION = 0x1209,
    AMBIENT = 0x1200,
    DIFFUSE = 0x1201,
    SPECULAR = 0x1202,
    SHININESS = 0x1601,
    EMISSION = 0x1600,
    POSITION = 0x1203,
    SPOT_DIRECTION = 0x1204,
    AMBIENT_AND_DIFFUSE = 0x1602,
    COLOR_INDEXES = 0x1603,
    LIGHT_MODEL_TWO_SIDE = 0x0B52,
    LIGHT_MODEL_LOCAL_VIEWER = 0x0B51,
    LIGHT_MODEL_AMBIENT = 0x0B53,
    FRONT_AND_BACK = 0x0408,
    SHADE_MODEL = 0x0B54,
    FLAT = 0x1D00,
    SMOOTH = 0x1D01,
    COLOR_MATERIAL = 0x0B57,
    COLOR_MATERIAL_FACE = 0x0B55,
    COLOR_MATERIAL_PARAMETER = 0x0B56,
    NORMALIZE = 0x0BA1,
    CLIP_PLANE0 = 0x3000,
    CLIP_PLANE1 = 0x3001,
    CLIP_PLANE2 = 0x3002,
    CLIP_PLANE3 = 0x3003,
    CLIP_PLANE4 = 0x3004,
    CLIP_PLANE5 = 0x3005,
    ACCUM_RED_BITS = 0x0D58,
    ACCUM_GREEN_BITS = 0x0D59,
    ACCUM_BLUE_BITS = 0x0D5A,
    ACCUM_ALPHA_BITS = 0x0D5B,
    ACCUM_CLEAR_VALUE = 0x0B80,
    ACCUM = 0x0100,
    ADD = 0x0104,
    LOAD = 0x0101,
    MULT = 0x0103,
    RETURN = 0x0102,
    ALPHA_TEST = 0x0BC0,
    ALPHA_TEST_REF = 0x0BC2,
    ALPHA_TEST_FUNC = 0x0BC1,
    BLEND = 0x0BE2,
    BLEND_SRC = 0x0BE1,
    BLEND_DST = 0x0BE0,
    ZERO = 0x0,
    ONE = 0x1,
    SRC_COLOR = 0x0300,
    ONE_MINUS_SRC_COLOR = 0x0301,
    DST_COLOR = 0x0306,
    ONE_MINUS_DST_COLOR = 0x0307,
    SRC_ALPHA = 0x0302,
    ONE_MINUS_SRC_ALPHA = 0x0303,
    DST_ALPHA = 0x0304,
    ONE_MINUS_DST_ALPHA = 0x0305,
    SRC_ALPHA_SATURATE = 0x0308,
    CONSTANT_COLOR = 0x8001,
    ONE_MINUS_CONSTANT_COLOR = 0x8002,
    CONSTANT_ALPHA = 0x8003,
    ONE_MINUS_CONSTANT_ALPHA = 0x8004,
    FEEDBACK = 0x1C01,
    RENDER = 0x1C00,
    SELECT = 0x1C02,
    // 2D = 0x0600,
    // 3D = 0x0601,
    // 3D_COLOR = 0x0602,
    // 3D_COLOR_TEXTURE = 0x0603,
    // 4D_COLOR_TEXTURE = 0x0604,
    POINT_TOKEN = 0x0701,
    LINE_TOKEN = 0x0702,
    LINE_RESET_TOKEN = 0x0707,
    POLYGON_TOKEN = 0x0703,
    BITMAP_TOKEN = 0x0704,
    DRAW_PIXEL_TOKEN = 0x0705,
    COPY_PIXEL_TOKEN = 0x0706,
    PASS_THROUGH_TOKEN = 0x0700,
    FEEDBACK_BUFFER_POINTER = 0x0DF0,
    FEEDBACK_BUFFER_SIZE = 0x0DF1,
    FEEDBACK_BUFFER_TYPE = 0x0DF2,
    SELECTION_BUFFER_POINTER = 0x0DF3,
    SELECTION_BUFFER_SIZE = 0x0DF4,
    FOG = 0x0B60,
    FOG_MODE = 0x0B65,
    FOG_DENSITY = 0x0B62,
    FOG_COLOR = 0x0B66,
    FOG_INDEX = 0x0B61,
    FOG_START = 0x0B63,
    FOG_END = 0x0B64,
    LINEAR = 0x2601,
    EXP = 0x0800,
    EXP2 = 0x0801,
    LOGIC_OP = 0x0BF1,
    INDEX_LOGIC_OP = 0x0BF1,
    COLOR_LOGIC_OP = 0x0BF2,
    LOGIC_OP_MODE = 0x0BF0,
    CLEAR = 0x1500,
    SET = 0x150F,
    COPY = 0x1503,
    COPY_INVERTED = 0x150C,
    NOOP = 0x1505,
    INVERT = 0x150A,
    AND = 0x1501,
    NAND = 0x150E,
    OR = 0x1507,
    NOR = 0x1508,
    XOR = 0x1506,
    EQUIV = 0x1509,
    AND_REVERSE = 0x1502,
    AND_INVERTED = 0x1504,
    OR_REVERSE = 0x150B,
    OR_INVERTED = 0x150D,
    STENCIL_TEST = 0x0B90,
    STENCIL_WRITEMASK = 0x0B98,
    STENCIL_BITS = 0x0D57,
    STENCIL_FUNC = 0x0B92,
    STENCIL_VALUE_MASK = 0x0B93,
    STENCIL_REF = 0x0B97,
    STENCIL_FAIL = 0x0B94,
    STENCIL_PASS_DEPTH_PASS = 0x0B96,
    STENCIL_PASS_DEPTH_FAIL = 0x0B95,
    STENCIL_CLEAR_VALUE = 0x0B91,
    STENCIL_INDEX = 0x1901,
    KEEP = 0x1E00,
    REPLACE = 0x1E01,
    INCR = 0x1E02,
    DECR = 0x1E03,
    NONE = 0x0,
    LEFT = 0x0406,
    RIGHT = 0x0407,
    FRONT_LEFT = 0x0400,
    FRONT_RIGHT = 0x0401,
    BACK_LEFT = 0x0402,
    BACK_RIGHT = 0x0403,
    AUX0 = 0x0409,
    AUX1 = 0x040A,
    AUX2 = 0x040B,
    AUX3 = 0x040C,
    COLOR_INDEX = 0x1900,
    RED = 0x1903,
    GREEN = 0x1904,
    BLUE = 0x1905,
    ALPHA = 0x1906,
    LUMINANCE = 0x1909,
    LUMINANCE_ALPHA = 0x190A,
    ALPHA_BITS = 0x0D55,
    RED_BITS = 0x0D52,
    GREEN_BITS = 0x0D53,
    BLUE_BITS = 0x0D54,
    INDEX_BITS = 0x0D51,
    SUBPIXEL_BITS = 0x0D50,
    AUX_BUFFERS = 0x0C00,
    READ_BUFFER = 0x0C02,
    DRAW_BUFFER = 0x0C01,
    DOUBLEBUFFER = 0x0C32,
    STEREO = 0x0C33,
    BITMAP = 0x1A00,
    COLOR = 0x1800,
    DEPTH = 0x1801,
    STENCIL = 0x1802,
    DITHER = 0x0BD0,
    RGB = 0x1907,
    RGBA = 0x1908,
    MAX_LIST_NESTING = 0x0B31,
    MAX_ATTRIB_STACK_DEPTH = 0x0D35,
    MAX_MODELVIEW_STACK_DEPTH = 0x0D36,
    MAX_NAME_STACK_DEPTH = 0x0D37,
    MAX_PROJECTION_STACK_DEPTH = 0x0D38,
    MAX_TEXTURE_STACK_DEPTH = 0x0D39,
    MAX_EVAL_ORDER = 0x0D30,
    MAX_LIGHTS = 0x0D31,
    MAX_CLIP_PLANES = 0x0D32,
    MAX_TEXTURE_SIZE = 0x0D33,
    MAX_PIXEL_MAP_TABLE = 0x0D34,
    MAX_VIEWPORT_DIMS = 0x0D3A,
    MAX_CLIENT_ATTRIB_STACK_DEPTH = 0x0D3B,
    ATTRIB_STACK_DEPTH = 0x0BB0,
    CLIENT_ATTRIB_STACK_DEPTH = 0x0BB1,
    COLOR_CLEAR_VALUE = 0x0C22,
    COLOR_WRITEMASK = 0x0C23,
    CURRENT_INDEX = 0x0B01,
    CURRENT_COLOR = 0x0B00,
    CURRENT_NORMAL = 0x0B02,
    CURRENT_RASTER_COLOR = 0x0B04,
    CURRENT_RASTER_DISTANCE = 0x0B09,
    CURRENT_RASTER_INDEX = 0x0B05,
    CURRENT_RASTER_POSITION = 0x0B07,
    CURRENT_RASTER_TEXTURE_COORDS = 0x0B06,
    CURRENT_RASTER_POSITION_VALID = 0x0B08,
    CURRENT_TEXTURE_COORDS = 0x0B03,
    INDEX_CLEAR_VALUE = 0x0C20,
    INDEX_MODE = 0x0C30,
    INDEX_WRITEMASK = 0x0C21,
    MODELVIEW_MATRIX = 0x0BA6,
    MODELVIEW_STACK_DEPTH = 0x0BA3,
    NAME_STACK_DEPTH = 0x0D70,
    PROJECTION_MATRIX = 0x0BA7,
    PROJECTION_STACK_DEPTH = 0x0BA4,
    RENDER_MODE = 0x0C40,
    RGBA_MODE = 0x0C31,
    TEXTURE_MATRIX = 0x0BA8,
    TEXTURE_STACK_DEPTH = 0x0BA5,
    VIEWPORT = 0x0BA2,
    AUTO_NORMAL = 0x0D80,
    MAP1_COLOR_4 = 0x0D90,
    MAP1_GRID_DOMAIN = 0x0DD0,
    MAP1_GRID_SEGMENTS = 0x0DD1,
    MAP1_INDEX = 0x0D91,
    MAP1_NORMAL = 0x0D92,
    MAP1_TEXTURE_COORD_1 = 0x0D93,
    MAP1_TEXTURE_COORD_2 = 0x0D94,
    MAP1_TEXTURE_COORD_3 = 0x0D95,
    MAP1_TEXTURE_COORD_4 = 0x0D96,
    MAP1_VERTEX_3 = 0x0D97,
    MAP1_VERTEX_4 = 0x0D98,
    MAP2_COLOR_4 = 0x0DB0,
    MAP2_GRID_DOMAIN = 0x0DD2,
    MAP2_GRID_SEGMENTS = 0x0DD3,
    MAP2_INDEX = 0x0DB1,
    MAP2_NORMAL = 0x0DB2,
    MAP2_TEXTURE_COORD_1 = 0x0DB3,
    MAP2_TEXTURE_COORD_2 = 0x0DB4,
    MAP2_TEXTURE_COORD_3 = 0x0DB5,
    MAP2_TEXTURE_COORD_4 = 0x0DB6,
    MAP2_VERTEX_3 = 0x0DB7,
    MAP2_VERTEX_4 = 0x0DB8,
    COEFF = 0x0A00,
    // DOMAIN = 0x0A02,
    ORDER = 0x0A01,
    FOG_HINT = 0x0C54,
    LINE_SMOOTH_HINT = 0x0C52,
    PERSPECTIVE_CORRECTION_HINT = 0x0C50,
    POINT_SMOOTH_HINT = 0x0C51,
    POLYGON_SMOOTH_HINT = 0x0C53,
    DONT_CARE = 0x1100,
    FASTEST = 0x1101,
    NICEST = 0x1102,
    SCISSOR_TEST = 0x0C11,
    SCISSOR_BOX = 0x0C10,
    MAP_COLOR = 0x0D10,
    MAP_STENCIL = 0x0D11,
    INDEX_SHIFT = 0x0D12,
    INDEX_OFFSET = 0x0D13,
    RED_SCALE = 0x0D14,
    RED_BIAS = 0x0D15,
    GREEN_SCALE = 0x0D18,
    GREEN_BIAS = 0x0D19,
    BLUE_SCALE = 0x0D1A,
    BLUE_BIAS = 0x0D1B,
    ALPHA_SCALE = 0x0D1C,
    ALPHA_BIAS = 0x0D1D,
    DEPTH_SCALE = 0x0D1E,
    DEPTH_BIAS = 0x0D1F,
    PIXEL_MAP_S_TO_S_SIZE = 0x0CB1,
    PIXEL_MAP_I_TO_I_SIZE = 0x0CB0,
    PIXEL_MAP_I_TO_R_SIZE = 0x0CB2,
    PIXEL_MAP_I_TO_G_SIZE = 0x0CB3,
    PIXEL_MAP_I_TO_B_SIZE = 0x0CB4,
    PIXEL_MAP_I_TO_A_SIZE = 0x0CB5,
    PIXEL_MAP_R_TO_R_SIZE = 0x0CB6,
    PIXEL_MAP_G_TO_G_SIZE = 0x0CB7,
    PIXEL_MAP_B_TO_B_SIZE = 0x0CB8,
    PIXEL_MAP_A_TO_A_SIZE = 0x0CB9,
    PIXEL_MAP_S_TO_S = 0x0C71,
    PIXEL_MAP_I_TO_I = 0x0C70,
    PIXEL_MAP_I_TO_R = 0x0C72,
    PIXEL_MAP_I_TO_G = 0x0C73,
    PIXEL_MAP_I_TO_B = 0x0C74,
    PIXEL_MAP_I_TO_A = 0x0C75,
    PIXEL_MAP_R_TO_R = 0x0C76,
    PIXEL_MAP_G_TO_G = 0x0C77,
    PIXEL_MAP_B_TO_B = 0x0C78,
    PIXEL_MAP_A_TO_A = 0x0C79,
    PACK_ALIGNMENT = 0x0D05,
    PACK_LSB_FIRST = 0x0D01,
    PACK_ROW_LENGTH = 0x0D02,
    PACK_SKIP_PIXELS = 0x0D04,
    PACK_SKIP_ROWS = 0x0D03,
    PACK_SWAP_BYTES = 0x0D00,
    UNPACK_ALIGNMENT = 0x0CF5,
    UNPACK_LSB_FIRST = 0x0CF1,
    UNPACK_ROW_LENGTH = 0x0CF2,
    UNPACK_SKIP_PIXELS = 0x0CF4,
    UNPACK_SKIP_ROWS = 0x0CF3,
    UNPACK_SWAP_BYTES = 0x0CF0,
    ZOOM_X = 0x0D16,
    ZOOM_Y = 0x0D17,
    TEXTURE_ENV = 0x2300,
    TEXTURE_ENV_MODE = 0x2200,
    TEXTURE_1D = 0x0DE0,
    TEXTURE_2D = 0x0DE1,
    TEXTURE_WRAP_S = 0x2802,
    TEXTURE_WRAP_T = 0x2803,
    TEXTURE_MAG_FILTER = 0x2800,
    TEXTURE_MIN_FILTER = 0x2801,
    TEXTURE_ENV_COLOR = 0x2201,
    TEXTURE_GEN_S = 0x0C60,
    TEXTURE_GEN_T = 0x0C61,
    TEXTURE_GEN_MODE = 0x2500,
    TEXTURE_BORDER_COLOR = 0x1004,
    TEXTURE_WIDTH = 0x1000,
    TEXTURE_HEIGHT = 0x1001,
    TEXTURE_BORDER = 0x1005,
    TEXTURE_COMPONENTS = 0x1003,
    TEXTURE_RED_SIZE = 0x805C,
    TEXTURE_GREEN_SIZE = 0x805D,
    TEXTURE_BLUE_SIZE = 0x805E,
    TEXTURE_ALPHA_SIZE = 0x805F,
    TEXTURE_LUMINANCE_SIZE = 0x8060,
    TEXTURE_INTENSITY_SIZE = 0x8061,
    NEAREST_MIPMAP_NEAREST = 0x2700,
    NEAREST_MIPMAP_LINEAR = 0x2702,
    LINEAR_MIPMAP_NEAREST = 0x2701,
    LINEAR_MIPMAP_LINEAR = 0x2703,
    OBJECT_LINEAR = 0x2401,
    OBJECT_PLANE = 0x2501,
    EYE_LINEAR = 0x2400,
    EYE_PLANE = 0x2502,
    SPHERE_MAP = 0x2402,
    DECAL = 0x2101,
    MODULATE = 0x2100,
    NEAREST = 0x2600,
    REPEAT = 0x2901,
    CLAMP = 0x2900,
    S = 0x2000,
    T = 0x2001,
    R = 0x2002,
    Q = 0x2003,
    TEXTURE_GEN_R = 0x0C62,
    TEXTURE_GEN_Q = 0x0C63,
    PROXY_TEXTURE_1D = 0x8063,
    PROXY_TEXTURE_2D = 0x8064,
    TEXTURE_PRIORITY = 0x8066,
    TEXTURE_RESIDENT = 0x8067,
    TEXTURE_BINDING_1D = 0x8068,
    TEXTURE_BINDING_2D = 0x8069,
    TEXTURE_INTERNAL_FORMAT = 0x1003,
    PACK_SKIP_IMAGES = 0x806B,
    PACK_IMAGE_HEIGHT = 0x806C,
    UNPACK_SKIP_IMAGES = 0x806D,
    UNPACK_IMAGE_HEIGHT = 0x806E,
    TEXTURE_3D = 0x806F,
    PROXY_TEXTURE_3D = 0x8070,
    TEXTURE_DEPTH = 0x8071,
    TEXTURE_WRAP_R = 0x8072,
    MAX_3D_TEXTURE_SIZE = 0x8073,
    TEXTURE_BINDING_3D = 0x806A,
    ALPHA4 = 0x803B,
    ALPHA8 = 0x803C,
    ALPHA12 = 0x803D,
    ALPHA16 = 0x803E,
    LUMINANCE4 = 0x803F,
    LUMINANCE8 = 0x8040,
    LUMINANCE12 = 0x8041,
    LUMINANCE16 = 0x8042,
    LUMINANCE4_ALPHA4 = 0x8043,
    LUMINANCE6_ALPHA2 = 0x8044,
    LUMINANCE8_ALPHA8 = 0x8045,
    LUMINANCE12_ALPHA4 = 0x8046,
    LUMINANCE12_ALPHA12 = 0x8047,
    LUMINANCE16_ALPHA16 = 0x8048,
    INTENSITY = 0x8049,
    INTENSITY4 = 0x804A,
    INTENSITY8 = 0x804B,
    INTENSITY12 = 0x804C,
    INTENSITY16 = 0x804D,
    R3_G3_B2 = 0x2A10,
    RGB4 = 0x804F,
    RGB5 = 0x8050,
    RGB8 = 0x8051,
    RGB10 = 0x8052,
    RGB12 = 0x8053,
    RGB16 = 0x8054,
    RGBA2 = 0x8055,
    RGBA4 = 0x8056,
    RGB5_A1 = 0x8057,
    RGBA8 = 0x8058,
    RGB10_A2 = 0x8059,
    RGBA12 = 0x805A,
    RGBA16 = 0x805B,
    VENDOR = 0x1F00,
    RENDERER = 0x1F01,
    VERSION = 0x1F02,
    EXTENSIONS = 0x1F03,
    NO_ERROR = 0x0,
    INVALID_VALUE = 0x0501,
    INVALID_ENUM = 0x0500,
    INVALID_OPERATION = 0x0502,
    STACK_OVERFLOW = 0x0503,
    STACK_UNDERFLOW = 0x0504,
    OUT_OF_MEMORY = 0x0505,
    RESCALE_NORMAL = 0x803A,
    CLAMP_TO_EDGE = 0x812F,
    MAX_ELEMENTS_VERTICES = 0x80E8,
    MAX_ELEMENTS_INDICES = 0x80E9,
    BGR = 0x80E0,
    BGRA = 0x80E1,
    UNSIGNED_BYTE_3_3_2 = 0x8032,
    UNSIGNED_BYTE_2_3_3_REV = 0x8362,
    UNSIGNED_SHORT_5_6_5 = 0x8363,
    UNSIGNED_SHORT_5_6_5_REV = 0x8364,
    UNSIGNED_SHORT_4_4_4_4 = 0x8033,
    UNSIGNED_SHORT_4_4_4_4_REV = 0x8365,
    UNSIGNED_SHORT_5_5_5_1 = 0x8034,
    UNSIGNED_SHORT_1_5_5_5_REV = 0x8366,
    UNSIGNED_INT_8_8_8_8 = 0x8035,
    UNSIGNED_INT_8_8_8_8_REV = 0x8367,
    UNSIGNED_INT_10_10_10_2 = 0x8036,
    UNSIGNED_INT_2_10_10_10_REV = 0x8368,
    LIGHT_MODEL_COLOR_CONTROL = 0x81F8,
    SINGLE_COLOR = 0x81F9,
    SEPARATE_SPECULAR_COLOR = 0x81FA,
    TEXTURE_MIN_LOD = 0x813A,
    TEXTURE_MAX_LOD = 0x813B,
    TEXTURE_BASE_LEVEL = 0x813C,
    TEXTURE_MAX_LEVEL = 0x813D,
    SMOOTH_POINT_SIZE_RANGE = 0x0B12,
    SMOOTH_POINT_SIZE_GRANULARITY = 0x0B13,
    SMOOTH_LINE_WIDTH_RANGE = 0x0B22,
    SMOOTH_LINE_WIDTH_GRANULARITY = 0x0B23,
    ALIASED_POINT_SIZE_RANGE = 0x846D,
    ALIASED_LINE_WIDTH_RANGE = 0x846E,
    COLOR_TABLE = 0x80D0,
    POST_CONVOLUTION_COLOR_TABLE = 0x80D1,
    POST_COLOR_MATRIX_COLOR_TABLE = 0x80D2,
    PROXY_COLOR_TABLE = 0x80D3,
    PROXY_POST_CONVOLUTION_COLOR_TABLE = 0x80D4,
    PROXY_POST_COLOR_MATRIX_COLOR_TABLE = 0x80D5,
    COLOR_TABLE_SCALE = 0x80D6,
    COLOR_TABLE_BIAS = 0x80D7,
    COLOR_TABLE_FORMAT = 0x80D8,
    COLOR_TABLE_WIDTH = 0x80D9,
    COLOR_TABLE_RED_SIZE = 0x80DA,
    COLOR_TABLE_GREEN_SIZE = 0x80DB,
    COLOR_TABLE_BLUE_SIZE = 0x80DC,
    COLOR_TABLE_ALPHA_SIZE = 0x80DD,
    COLOR_TABLE_LUMINANCE_SIZE = 0x80DE,
    COLOR_TABLE_INTENSITY_SIZE = 0x80DF,
    CONVOLUTION_1D = 0x8010,
    CONVOLUTION_2D = 0x8011,
    SEPARABLE_2D = 0x8012,
    CONVOLUTION_BORDER_MODE = 0x8013,
    CONVOLUTION_FILTER_SCALE = 0x8014,
    CONVOLUTION_FILTER_BIAS = 0x8015,
    REDUCE = 0x8016,
    CONVOLUTION_FORMAT = 0x8017,
    CONVOLUTION_WIDTH = 0x8018,
    CONVOLUTION_HEIGHT = 0x8019,
    MAX_CONVOLUTION_WIDTH = 0x801A,
    MAX_CONVOLUTION_HEIGHT = 0x801B,
    POST_CONVOLUTION_RED_SCALE = 0x801C,
    POST_CONVOLUTION_GREEN_SCALE = 0x801D,
    POST_CONVOLUTION_BLUE_SCALE = 0x801E,
    POST_CONVOLUTION_ALPHA_SCALE = 0x801F,
    POST_CONVOLUTION_RED_BIAS = 0x8020,
    POST_CONVOLUTION_GREEN_BIAS = 0x8021,
    POST_CONVOLUTION_BLUE_BIAS = 0x8022,
    POST_CONVOLUTION_ALPHA_BIAS = 0x8023,
    CONSTANT_BORDER = 0x8151,
    REPLICATE_BORDER = 0x8153,
    CONVOLUTION_BORDER_COLOR = 0x8154,
    COLOR_MATRIX = 0x80B1,
    COLOR_MATRIX_STACK_DEPTH = 0x80B2,
    MAX_COLOR_MATRIX_STACK_DEPTH = 0x80B3,
    POST_COLOR_MATRIX_RED_SCALE = 0x80B4,
    POST_COLOR_MATRIX_GREEN_SCALE = 0x80B5,
    POST_COLOR_MATRIX_BLUE_SCALE = 0x80B6,
    POST_COLOR_MATRIX_ALPHA_SCALE = 0x80B7,
    POST_COLOR_MATRIX_RED_BIAS = 0x80B8,
    POST_COLOR_MATRIX_GREEN_BIAS = 0x80B9,
    POST_COLOR_MATRIX_BLUE_BIAS = 0x80BA,
    POST_COLOR_MATRIX_ALPHA_BIAS = 0x80BB,
    HISTOGRAM = 0x8024,
    PROXY_HISTOGRAM = 0x8025,
    HISTOGRAM_WIDTH = 0x8026,
    HISTOGRAM_FORMAT = 0x8027,
    HISTOGRAM_RED_SIZE = 0x8028,
    HISTOGRAM_GREEN_SIZE = 0x8029,
    HISTOGRAM_BLUE_SIZE = 0x802A,
    HISTOGRAM_ALPHA_SIZE = 0x802B,
    HISTOGRAM_LUMINANCE_SIZE = 0x802C,
    HISTOGRAM_SINK = 0x802D,
    MINMAX = 0x802E,
    MINMAX_FORMAT = 0x802F,
    MINMAX_SINK = 0x8030,
    TABLE_TOO_LARGE = 0x8031,
    BLEND_EQUATION = 0x8009,
    MIN = 0x8007,
    MAX = 0x8008,
    FUNC_ADD = 0x8006,
    FUNC_SUBTRACT = 0x800A,
    FUNC_REVERSE_SUBTRACT = 0x800B,
    BLEND_COLOR = 0x8005,
    CURRENT_BIT = 0x00000001,
    POINT_BIT = 0x00000002,
    LINE_BIT = 0x00000004,
    POLYGON_BIT = 0x00000008,
    POLYGON_STIPPLE_BIT = 0x00000010,
    PIXEL_MODE_BIT = 0x00000020,
    LIGHTING_BIT = 0x00000040,
    FOG_BIT = 0x00000080,
    DEPTH_BUFFER_BIT = 0x00000100,
    ACCUM_BUFFER_BIT = 0x00000200,
    STENCIL_BUFFER_BIT = 0x00000400,
    VIEWPORT_BIT = 0x00000800,
    TRANSFORM_BIT = 0x00001000,
    ENABLE_BIT = 0x00002000,
    COLOR_BUFFER_BIT = 0x00004000,
    HINT_BIT = 0x00008000,
    EVAL_BIT = 0x00010000,
    LIST_BIT = 0x00020000,
    TEXTURE_BIT = 0x00040000,
    SCISSOR_BIT = 0x00080000,
    ALL_ATTRIB_BITS = 0x000FFFFF,
    CLIENT_PIXEL_STORE_BIT = 0x00000001,
    CLIENT_VERTEX_ARRAY_BIT = 0x00000002,
    ALL_CLIENT_ATTRIB_BITS = 0xFFFFFFFF,
    ARB_MULTITEXTURE = 1,
    TEXTURE0_ARB = 0x84C0,
    TEXTURE1_ARB = 0x84C1,
    TEXTURE2_ARB = 0x84C2,
    TEXTURE3_ARB = 0x84C3,
    TEXTURE4_ARB = 0x84C4,
    TEXTURE5_ARB = 0x84C5,
    TEXTURE6_ARB = 0x84C6,
    TEXTURE7_ARB = 0x84C7,
    TEXTURE8_ARB = 0x84C8,
    TEXTURE9_ARB = 0x84C9,
    TEXTURE10_ARB = 0x84CA,
    TEXTURE11_ARB = 0x84CB,
    TEXTURE12_ARB = 0x84CC,
    TEXTURE13_ARB = 0x84CD,
    TEXTURE14_ARB = 0x84CE,
    TEXTURE15_ARB = 0x84CF,
    TEXTURE16_ARB = 0x84D0,
    TEXTURE17_ARB = 0x84D1,
    TEXTURE18_ARB = 0x84D2,
    TEXTURE19_ARB = 0x84D3,
    TEXTURE20_ARB = 0x84D4,
    TEXTURE21_ARB = 0x84D5,
    TEXTURE22_ARB = 0x84D6,
    TEXTURE23_ARB = 0x84D7,
    TEXTURE24_ARB = 0x84D8,
    TEXTURE25_ARB = 0x84D9,
    TEXTURE26_ARB = 0x84DA,
    TEXTURE27_ARB = 0x84DB,
    TEXTURE28_ARB = 0x84DC,
    TEXTURE29_ARB = 0x84DD,
    TEXTURE30_ARB = 0x84DE,
    TEXTURE31_ARB = 0x84DF,
    ACTIVE_TEXTURE_ARB = 0x84E0,
    CLIENT_ACTIVE_TEXTURE_ARB = 0x84E1,
    MAX_TEXTURE_UNITS_ARB = 0x84E2,
    EXT_ABGR = 1,
    ABGR_EXT = 0x8000,
    EXT_BLEND_COLOR = 1,
    CONSTANT_COLOR_EXT = 0x8001,
    ONE_MINUS_CONSTANT_COLOR_EXT = 0x8002,
    CONSTANT_ALPHA_EXT = 0x8003,
    ONE_MINUS_CONSTANT_ALPHA_EXT = 0x8004,
    BLEND_COLOR_EXT = 0x8005,
    EXT_POLYGON_OFFSET = 1,
    POLYGON_OFFSET_EXT = 0x8037,
    POLYGON_OFFSET_FACTOR_EXT = 0x8038,
    POLYGON_OFFSET_BIAS_EXT = 0x8039,
    EXT_TEXTURE3D = 1,
    PACK_SKIP_IMAGES_EXT = 0x806B,
    PACK_IMAGE_HEIGHT_EXT = 0x806C,
    UNPACK_SKIP_IMAGES_EXT = 0x806D,
    UNPACK_IMAGE_HEIGHT_EXT = 0x806E,
    TEXTURE_3D_EXT = 0x806F,
    PROXY_TEXTURE_3D_EXT = 0x8070,
    TEXTURE_DEPTH_EXT = 0x8071,
    TEXTURE_WRAP_R_EXT = 0x8072,
    MAX_3D_TEXTURE_SIZE_EXT = 0x8073,
    TEXTURE_3D_BINDING_EXT = 0x806A,
    EXT_TEXTURE_OBJECT = 1,
    TEXTURE_PRIORITY_EXT = 0x8066,
    TEXTURE_RESIDENT_EXT = 0x8067,
    TEXTURE_1D_BINDING_EXT = 0x8068,
    TEXTURE_2D_BINDING_EXT = 0x8069,
    EXT_RESCALE_NORMAL = 1,
    RESCALE_NORMAL_EXT = 0x803A,
    EXT_VERTEX_ARRAY = 1,
    VERTEX_ARRAY_EXT = 0x8074,
    NORMAL_ARRAY_EXT = 0x8075,
    COLOR_ARRAY_EXT = 0x8076,
    INDEX_ARRAY_EXT = 0x8077,
    TEXTURE_COORD_ARRAY_EXT = 0x8078,
    EDGE_FLAG_ARRAY_EXT = 0x8079,
    VERTEX_ARRAY_SIZE_EXT = 0x807A,
    VERTEX_ARRAY_TYPE_EXT = 0x807B,
    VERTEX_ARRAY_STRIDE_EXT = 0x807C,
    VERTEX_ARRAY_COUNT_EXT = 0x807D,
    NORMAL_ARRAY_TYPE_EXT = 0x807E,
    NORMAL_ARRAY_STRIDE_EXT = 0x807F,
    NORMAL_ARRAY_COUNT_EXT = 0x8080,
    COLOR_ARRAY_SIZE_EXT = 0x8081,
    COLOR_ARRAY_TYPE_EXT = 0x8082,
    COLOR_ARRAY_STRIDE_EXT = 0x8083,
    COLOR_ARRAY_COUNT_EXT = 0x8084,
    INDEX_ARRAY_TYPE_EXT = 0x8085,
    INDEX_ARRAY_STRIDE_EXT = 0x8086,
    INDEX_ARRAY_COUNT_EXT = 0x8087,
    TEXTURE_COORD_ARRAY_SIZE_EXT = 0x8088,
    TEXTURE_COORD_ARRAY_TYPE_EXT = 0x8089,
    TEXTURE_COORD_ARRAY_STRIDE_EXT = 0x808A,
    TEXTURE_COORD_ARRAY_COUNT_EXT = 0x808B,
    EDGE_FLAG_ARRAY_STRIDE_EXT = 0x808C,
    EDGE_FLAG_ARRAY_COUNT_EXT = 0x808D,
    VERTEX_ARRAY_POINTER_EXT = 0x808E,
    NORMAL_ARRAY_POINTER_EXT = 0x808F,
    COLOR_ARRAY_POINTER_EXT = 0x8090,
    INDEX_ARRAY_POINTER_EXT = 0x8091,
    TEXTURE_COORD_ARRAY_POINTER_EXT = 0x8092,
    EDGE_FLAG_ARRAY_POINTER_EXT = 0x8093,
    SGIS_TEXTURE_EDGE_CLAMP = 1,
    CLAMP_TO_EDGE_SGIS = 0x812F,
    EXT_BLEND_MINMAX = 1,
    FUNC_ADD_EXT = 0x8006,
    MIN_EXT = 0x8007,
    MAX_EXT = 0x8008,
    BLEND_EQUATION_EXT = 0x8009,
    EXT_BLEND_SUBTRACT = 1,
    FUNC_SUBTRACT_EXT = 0x800A,
    FUNC_REVERSE_SUBTRACT_EXT = 0x800B,
    EXT_BLEND_LOGIC_OP = 1,
    EXT_POINT_PARAMETERS = 1,
    POINT_SIZE_MIN_EXT = 0x8126,
    POINT_SIZE_MAX_EXT = 0x8127,
    POINT_FADE_THRESHOLD_SIZE_EXT = 0x8128,
    DISTANCE_ATTENUATION_EXT = 0x8129,
    EXT_PALETTED_TEXTURE = 1,
    TABLE_TOO_LARGE_EXT = 0x8031,
    COLOR_TABLE_FORMAT_EXT = 0x80D8,
    COLOR_TABLE_WIDTH_EXT = 0x80D9,
    COLOR_TABLE_RED_SIZE_EXT = 0x80DA,
    COLOR_TABLE_GREEN_SIZE_EXT = 0x80DB,
    COLOR_TABLE_BLUE_SIZE_EXT = 0x80DC,
    COLOR_TABLE_ALPHA_SIZE_EXT = 0x80DD,
    COLOR_TABLE_LUMINANCE_SIZE_EXT = 0x80DE,
    COLOR_TABLE_INTENSITY_SIZE_EXT = 0x80DF,
    TEXTURE_INDEX_SIZE_EXT = 0x80ED,
    COLOR_INDEX1_EXT = 0x80E2,
    COLOR_INDEX2_EXT = 0x80E3,
    COLOR_INDEX4_EXT = 0x80E4,
    COLOR_INDEX8_EXT = 0x80E5,
    COLOR_INDEX12_EXT = 0x80E6,
    COLOR_INDEX16_EXT = 0x80E7,
    EXT_CLIP_VOLUME_HINT = 1,
    CLIP_VOLUME_CLIPPING_HINT_EXT = 0x80F0,
    EXT_COMPILED_VERTEX_ARRAY = 1,
    ARRAY_ELEMENT_LOCK_FIRST_EXT = 0x81A8,
    ARRAY_ELEMENT_LOCK_COUNT_EXT = 0x81A9,
    HP_OCCLUSION_TEST = 1,
    OCCLUSION_TEST_HP = 0x8165,
    OCCLUSION_TEST_RESULT_HP = 0x8166,
    EXT_SHARED_TEXTURE_PALETTE = 1,
    SHARED_TEXTURE_PALETTE_EXT = 0x81FB,
    EXT_STENCIL_WRAP = 1,
    INCR_WRAP_EXT = 0x8507,
    DECR_WRAP_EXT = 0x8508,
    NV_TEXGEN_REFLECTION = 1,
    NORMAL_MAP_NV = 0x8511,
    REFLECTION_MAP_NV = 0x8512,
    EXT_TEXTURE_ENV_ADD = 1,
    MESA_WINDOW_POS = 1,
    MESA_RESIZE_BUFFERS = 1,
    EXT_TEXTURE_ENV_DOT3 = 1,
    DOT3_RGB_EXT = 0x8740,
    DOT3_RGBA_EXT = 0x8741
  };
};

%extend GL
{
  static GLConfig& config()
  {
    return GL::config;
  }
};


%{
  enum
  {
    GL_VERSION_1_1 = 1,
    GL_VERSION_1_2 = 1,
    GL_FALSE = 0x0,
    GL_TRUE = 0x1,
    GL_BYTE = 0x1400,
    GL_UNSIGNED_BYTE = 0x1401,
    GL_SHORT = 0x1402,
    GL_UNSIGNED_SHORT = 0x1403,
    GL_INT = 0x1404,
    GL_UNSIGNED_INT = 0x1405,
    GL_FLOAT = 0x1406,
    GL_DOUBLE = 0x140A,
    GL_2_BYTES = 0x1407,
    GL_3_BYTES = 0x1408,
    GL_4_BYTES = 0x1409,
    GL_POINTS = 0x0000,
    GL_LINES = 0x0001,
    GL_LINE_LOOP = 0x0002,
    GL_LINE_STRIP = 0x0003,
    GL_TRIANGLES = 0x0004,
    GL_TRIANGLE_STRIP = 0x0005,
    GL_TRIANGLE_FAN = 0x0006,
    GL_QUADS = 0x0007,
    GL_QUAD_STRIP = 0x0008,
    GL_POLYGON = 0x0009,
    GL_VERTEX_ARRAY = 0x8074,
    GL_NORMAL_ARRAY = 0x8075,
    GL_COLOR_ARRAY = 0x8076,
    GL_INDEX_ARRAY = 0x8077,
    GL_TEXTURE_COORD_ARRAY = 0x8078,
    GL_EDGE_FLAG_ARRAY = 0x8079,
    GL_VERTEX_ARRAY_SIZE = 0x807A,
    GL_VERTEX_ARRAY_TYPE = 0x807B,
    GL_VERTEX_ARRAY_STRIDE = 0x807C,
    GL_NORMAL_ARRAY_TYPE = 0x807E,
    GL_NORMAL_ARRAY_STRIDE = 0x807F,
    GL_COLOR_ARRAY_SIZE = 0x8081,
    GL_COLOR_ARRAY_TYPE = 0x8082,
    GL_COLOR_ARRAY_STRIDE = 0x8083,
    GL_INDEX_ARRAY_TYPE = 0x8085,
    GL_INDEX_ARRAY_STRIDE = 0x8086,
    GL_TEXTURE_COORD_ARRAY_SIZE = 0x8088,
    GL_TEXTURE_COORD_ARRAY_TYPE = 0x8089,
    GL_TEXTURE_COORD_ARRAY_STRIDE = 0x808A,
    GL_EDGE_FLAG_ARRAY_STRIDE = 0x808C,
    GL_VERTEX_ARRAY_POINTER = 0x808E,
    GL_NORMAL_ARRAY_POINTER = 0x808F,
    GL_COLOR_ARRAY_POINTER = 0x8090,
    GL_INDEX_ARRAY_POINTER = 0x8091,
    GL_TEXTURE_COORD_ARRAY_POINTER = 0x8092,
    GL_EDGE_FLAG_ARRAY_POINTER = 0x8093,
    GL_V2F = 0x2A20,
    GL_V3F = 0x2A21,
    GL_C4UB_V2F = 0x2A22,
    GL_C4UB_V3F = 0x2A23,
    GL_C3F_V3F = 0x2A24,
    GL_N3F_V3F = 0x2A25,
    GL_C4F_N3F_V3F = 0x2A26,
    GL_T2F_V3F = 0x2A27,
    GL_T4F_V4F = 0x2A28,
    GL_T2F_C4UB_V3F = 0x2A29,
    GL_T2F_C3F_V3F = 0x2A2A,
    GL_T2F_N3F_V3F = 0x2A2B,
    GL_T2F_C4F_N3F_V3F = 0x2A2C,
    GL_T4F_C4F_N3F_V4F = 0x2A2D,
    GL_MATRIX_MODE = 0x0BA0,
    GL_MODELVIEW = 0x1700,
    GL_PROJECTION = 0x1701,
    GL_TEXTURE = 0x1702,
    GL_POINT_SMOOTH = 0x0B10,
    GL_POINT_SIZE = 0x0B11,
    GL_POINT_SIZE_GRANULARITY = 0x0B13,
    GL_POINT_SIZE_RANGE = 0x0B12,
    GL_LINE_SMOOTH = 0x0B20,
    GL_LINE_STIPPLE = 0x0B24,
    GL_LINE_STIPPLE_PATTERN = 0x0B25,
    GL_LINE_STIPPLE_REPEAT = 0x0B26,
    GL_LINE_WIDTH = 0x0B21,
    GL_LINE_WIDTH_GRANULARITY = 0x0B23,
    GL_LINE_WIDTH_RANGE = 0x0B22,
    GL_POINT = 0x1B00,
    GL_LINE = 0x1B01,
    GL_FILL = 0x1B02,
    GL_CW = 0x0900,
    GL_CCW = 0x0901,
    GL_FRONT = 0x0404,
    GL_BACK = 0x0405,
    GL_POLYGON_MODE = 0x0B40,
    GL_POLYGON_SMOOTH = 0x0B41,
    GL_POLYGON_STIPPLE = 0x0B42,
    GL_EDGE_FLAG = 0x0B43,
    GL_CULL_FACE = 0x0B44,
    GL_CULL_FACE_MODE = 0x0B45,
    GL_FRONT_FACE = 0x0B46,
    GL_POLYGON_OFFSET_FACTOR = 0x8038,
    GL_POLYGON_OFFSET_UNITS = 0x2A00,
    GL_POLYGON_OFFSET_POINT = 0x2A01,
    GL_POLYGON_OFFSET_LINE = 0x2A02,
    GL_POLYGON_OFFSET_FILL = 0x8037,
    GL_COMPILE = 0x1300,
    GL_COMPILE_AND_EXECUTE = 0x1301,
    GL_LIST_BASE = 0x0B32,
    GL_LIST_INDEX = 0x0B33,
    GL_LIST_MODE = 0x0B30,
    GL_NEVER = 0x0200,
    GL_LESS = 0x0201,
    GL_EQUAL = 0x0202,
    GL_LEQUAL = 0x0203,
    GL_GREATER = 0x0204,
    GL_NOTEQUAL = 0x0205,
    GL_GEQUAL = 0x0206,
    GL_ALWAYS = 0x0207,
    GL_DEPTH_TEST = 0x0B71,
    GL_DEPTH_BITS = 0x0D56,
    GL_DEPTH_CLEAR_VALUE = 0x0B73,
    GL_DEPTH_FUNC = 0x0B74,
    GL_DEPTH_RANGE = 0x0B70,
    GL_DEPTH_WRITEMASK = 0x0B72,
    GL_DEPTH_COMPONENT = 0x1902,
    GL_LIGHTING = 0x0B50,
    GL_LIGHT0 = 0x4000,
    GL_LIGHT1 = 0x4001,
    GL_LIGHT2 = 0x4002,
    GL_LIGHT3 = 0x4003,
    GL_LIGHT4 = 0x4004,
    GL_LIGHT5 = 0x4005,
    GL_LIGHT6 = 0x4006,
    GL_LIGHT7 = 0x4007,
    GL_SPOT_EXPONENT = 0x1205,
    GL_SPOT_CUTOFF = 0x1206,
    GL_CONSTANT_ATTENUATION = 0x1207,
    GL_LINEAR_ATTENUATION = 0x1208,
    GL_QUADRATIC_ATTENUATION = 0x1209,
    GL_AMBIENT = 0x1200,
    GL_DIFFUSE = 0x1201,
    GL_SPECULAR = 0x1202,
    GL_SHININESS = 0x1601,
    GL_EMISSION = 0x1600,
    GL_POSITION = 0x1203,
    GL_SPOT_DIRECTION = 0x1204,
    GL_AMBIENT_AND_DIFFUSE = 0x1602,
    GL_COLOR_INDEXES = 0x1603,
    GL_LIGHT_MODEL_TWO_SIDE = 0x0B52,
    GL_LIGHT_MODEL_LOCAL_VIEWER = 0x0B51,
    GL_LIGHT_MODEL_AMBIENT = 0x0B53,
    GL_FRONT_AND_BACK = 0x0408,
    GL_SHADE_MODEL = 0x0B54,
    GL_FLAT = 0x1D00,
    GL_SMOOTH = 0x1D01,
    GL_COLOR_MATERIAL = 0x0B57,
    GL_COLOR_MATERIAL_FACE = 0x0B55,
    GL_COLOR_MATERIAL_PARAMETER = 0x0B56,
    GL_NORMALIZE = 0x0BA1,
    GL_CLIP_PLANE0 = 0x3000,
    GL_CLIP_PLANE1 = 0x3001,
    GL_CLIP_PLANE2 = 0x3002,
    GL_CLIP_PLANE3 = 0x3003,
    GL_CLIP_PLANE4 = 0x3004,
    GL_CLIP_PLANE5 = 0x3005,
    GL_ACCUM_RED_BITS = 0x0D58,
    GL_ACCUM_GREEN_BITS = 0x0D59,
    GL_ACCUM_BLUE_BITS = 0x0D5A,
    GL_ACCUM_ALPHA_BITS = 0x0D5B,
    GL_ACCUM_CLEAR_VALUE = 0x0B80,
    GL_ACCUM = 0x0100,
    GL_ADD = 0x0104,
    GL_LOAD = 0x0101,
    GL_MULT = 0x0103,
    GL_RETURN = 0x0102,
    GL_ALPHA_TEST = 0x0BC0,
    GL_ALPHA_TEST_REF = 0x0BC2,
    GL_ALPHA_TEST_FUNC = 0x0BC1,
    GL_BLEND = 0x0BE2,
    GL_BLEND_SRC = 0x0BE1,
    GL_BLEND_DST = 0x0BE0,
    GL_ZERO = 0x0,
    GL_ONE = 0x1,
    GL_SRC_COLOR = 0x0300,
    GL_ONE_MINUS_SRC_COLOR = 0x0301,
    GL_DST_COLOR = 0x0306,
    GL_ONE_MINUS_DST_COLOR = 0x0307,
    GL_SRC_ALPHA = 0x0302,
    GL_ONE_MINUS_SRC_ALPHA = 0x0303,
    GL_DST_ALPHA = 0x0304,
    GL_ONE_MINUS_DST_ALPHA = 0x0305,
    GL_SRC_ALPHA_SATURATE = 0x0308,
    GL_CONSTANT_COLOR = 0x8001,
    GL_ONE_MINUS_CONSTANT_COLOR = 0x8002,
    GL_CONSTANT_ALPHA = 0x8003,
    GL_ONE_MINUS_CONSTANT_ALPHA = 0x8004,
    GL_FEEDBACK = 0x1C01,
    GL_RENDER = 0x1C00,
    GL_SELECT = 0x1C02,
    GL_2D = 0x0600,
    GL_3D = 0x0601,
    GL_3D_COLOR = 0x0602,
    GL_3D_COLOR_TEXTURE = 0x0603,
    GL_4D_COLOR_TEXTURE = 0x0604,
    GL_POINT_TOKEN = 0x0701,
    GL_LINE_TOKEN = 0x0702,
    GL_LINE_RESET_TOKEN = 0x0707,
    GL_POLYGON_TOKEN = 0x0703,
    GL_BITMAP_TOKEN = 0x0704,
    GL_DRAW_PIXEL_TOKEN = 0x0705,
    GL_COPY_PIXEL_TOKEN = 0x0706,
    GL_PASS_THROUGH_TOKEN = 0x0700,
    GL_FEEDBACK_BUFFER_POINTER = 0x0DF0,
    GL_FEEDBACK_BUFFER_SIZE = 0x0DF1,
    GL_FEEDBACK_BUFFER_TYPE = 0x0DF2,
    GL_SELECTION_BUFFER_POINTER = 0x0DF3,
    GL_SELECTION_BUFFER_SIZE = 0x0DF4,
    GL_FOG = 0x0B60,
    GL_FOG_MODE = 0x0B65,
    GL_FOG_DENSITY = 0x0B62,
    GL_FOG_COLOR = 0x0B66,
    GL_FOG_INDEX = 0x0B61,
    GL_FOG_START = 0x0B63,
    GL_FOG_END = 0x0B64,
    GL_LINEAR = 0x2601,
    GL_EXP = 0x0800,
    GL_EXP2 = 0x0801,
    GL_LOGIC_OP = 0x0BF1,
    GL_INDEX_LOGIC_OP = 0x0BF1,
    GL_COLOR_LOGIC_OP = 0x0BF2,
    GL_LOGIC_OP_MODE = 0x0BF0,
    GL_CLEAR = 0x1500,
    GL_SET = 0x150F,
    GL_COPY = 0x1503,
    GL_COPY_INVERTED = 0x150C,
    GL_NOOP = 0x1505,
    GL_INVERT = 0x150A,
    GL_AND = 0x1501,
    GL_NAND = 0x150E,
    GL_OR = 0x1507,
    GL_NOR = 0x1508,
    GL_XOR = 0x1506,
    GL_EQUIV = 0x1509,
    GL_AND_REVERSE = 0x1502,
    GL_AND_INVERTED = 0x1504,
    GL_OR_REVERSE = 0x150B,
    GL_OR_INVERTED = 0x150D,
    GL_STENCIL_TEST = 0x0B90,
    GL_STENCIL_WRITEMASK = 0x0B98,
    GL_STENCIL_BITS = 0x0D57,
    GL_STENCIL_FUNC = 0x0B92,
    GL_STENCIL_VALUE_MASK = 0x0B93,
    GL_STENCIL_REF = 0x0B97,
    GL_STENCIL_FAIL = 0x0B94,
    GL_STENCIL_PASS_DEPTH_PASS = 0x0B96,
    GL_STENCIL_PASS_DEPTH_FAIL = 0x0B95,
    GL_STENCIL_CLEAR_VALUE = 0x0B91,
    GL_STENCIL_INDEX = 0x1901,
    GL_KEEP = 0x1E00,
    GL_REPLACE = 0x1E01,
    GL_INCR = 0x1E02,
    GL_DECR = 0x1E03,
    GL_NONE = 0x0,
    GL_LEFT = 0x0406,
    GL_RIGHT = 0x0407,
    GL_FRONT_LEFT = 0x0400,
    GL_FRONT_RIGHT = 0x0401,
    GL_BACK_LEFT = 0x0402,
    GL_BACK_RIGHT = 0x0403,
    GL_AUX0 = 0x0409,
    GL_AUX1 = 0x040A,
    GL_AUX2 = 0x040B,
    GL_AUX3 = 0x040C,
    GL_COLOR_INDEX = 0x1900,
    GL_RED = 0x1903,
    GL_GREEN = 0x1904,
    GL_BLUE = 0x1905,
    GL_ALPHA = 0x1906,
    GL_LUMINANCE = 0x1909,
    GL_LUMINANCE_ALPHA = 0x190A,
    GL_ALPHA_BITS = 0x0D55,
    GL_RED_BITS = 0x0D52,
    GL_GREEN_BITS = 0x0D53,
    GL_BLUE_BITS = 0x0D54,
    GL_INDEX_BITS = 0x0D51,
    GL_SUBPIXEL_BITS = 0x0D50,
    GL_AUX_BUFFERS = 0x0C00,
    GL_READ_BUFFER = 0x0C02,
    GL_DRAW_BUFFER = 0x0C01,
    GL_DOUBLEBUFFER = 0x0C32,
    GL_STEREO = 0x0C33,
    GL_BITMAP = 0x1A00,
    GL_COLOR = 0x1800,
    GL_DEPTH = 0x1801,
    GL_STENCIL = 0x1802,
    GL_DITHER = 0x0BD0,
    GL_RGB = 0x1907,
    GL_RGBA = 0x1908,
    GL_MAX_LIST_NESTING = 0x0B31,
    GL_MAX_ATTRIB_STACK_DEPTH = 0x0D35,
    GL_MAX_MODELVIEW_STACK_DEPTH = 0x0D36,
    GL_MAX_NAME_STACK_DEPTH = 0x0D37,
    GL_MAX_PROJECTION_STACK_DEPTH = 0x0D38,
    GL_MAX_TEXTURE_STACK_DEPTH = 0x0D39,
    GL_MAX_EVAL_ORDER = 0x0D30,
    GL_MAX_LIGHTS = 0x0D31,
    GL_MAX_CLIP_PLANES = 0x0D32,
    GL_MAX_TEXTURE_SIZE = 0x0D33,
    GL_MAX_PIXEL_MAP_TABLE = 0x0D34,
    GL_MAX_VIEWPORT_DIMS = 0x0D3A,
    GL_MAX_CLIENT_ATTRIB_STACK_DEPTH = 0x0D3B,
    GL_ATTRIB_STACK_DEPTH = 0x0BB0,
    GL_CLIENT_ATTRIB_STACK_DEPTH = 0x0BB1,
    GL_COLOR_CLEAR_VALUE = 0x0C22,
    GL_COLOR_WRITEMASK = 0x0C23,
    GL_CURRENT_INDEX = 0x0B01,
    GL_CURRENT_COLOR = 0x0B00,
    GL_CURRENT_NORMAL = 0x0B02,
    GL_CURRENT_RASTER_COLOR = 0x0B04,
    GL_CURRENT_RASTER_DISTANCE = 0x0B09,
    GL_CURRENT_RASTER_INDEX = 0x0B05,
    GL_CURRENT_RASTER_POSITION = 0x0B07,
    GL_CURRENT_RASTER_TEXTURE_COORDS = 0x0B06,
    GL_CURRENT_RASTER_POSITION_VALID = 0x0B08,
    GL_CURRENT_TEXTURE_COORDS = 0x0B03,
    GL_INDEX_CLEAR_VALUE = 0x0C20,
    GL_INDEX_MODE = 0x0C30,
    GL_INDEX_WRITEMASK = 0x0C21,
    GL_MODELVIEW_MATRIX = 0x0BA6,
    GL_MODELVIEW_STACK_DEPTH = 0x0BA3,
    GL_NAME_STACK_DEPTH = 0x0D70,
    GL_PROJECTION_MATRIX = 0x0BA7,
    GL_PROJECTION_STACK_DEPTH = 0x0BA4,
    GL_RENDER_MODE = 0x0C40,
    GL_RGBA_MODE = 0x0C31,
    GL_TEXTURE_MATRIX = 0x0BA8,
    GL_TEXTURE_STACK_DEPTH = 0x0BA5,
    GL_VIEWPORT = 0x0BA2,
    GL_AUTO_NORMAL = 0x0D80,
    GL_MAP1_COLOR_4 = 0x0D90,
    GL_MAP1_GRID_DOMAIN = 0x0DD0,
    GL_MAP1_GRID_SEGMENTS = 0x0DD1,
    GL_MAP1_INDEX = 0x0D91,
    GL_MAP1_NORMAL = 0x0D92,
    GL_MAP1_TEXTURE_COORD_1 = 0x0D93,
    GL_MAP1_TEXTURE_COORD_2 = 0x0D94,
    GL_MAP1_TEXTURE_COORD_3 = 0x0D95,
    GL_MAP1_TEXTURE_COORD_4 = 0x0D96,
    GL_MAP1_VERTEX_3 = 0x0D97,
    GL_MAP1_VERTEX_4 = 0x0D98,
    GL_MAP2_COLOR_4 = 0x0DB0,
    GL_MAP2_GRID_DOMAIN = 0x0DD2,
    GL_MAP2_GRID_SEGMENTS = 0x0DD3,
    GL_MAP2_INDEX = 0x0DB1,
    GL_MAP2_NORMAL = 0x0DB2,
    GL_MAP2_TEXTURE_COORD_1 = 0x0DB3,
    GL_MAP2_TEXTURE_COORD_2 = 0x0DB4,
    GL_MAP2_TEXTURE_COORD_3 = 0x0DB5,
    GL_MAP2_TEXTURE_COORD_4 = 0x0DB6,
    GL_MAP2_VERTEX_3 = 0x0DB7,
    GL_MAP2_VERTEX_4 = 0x0DB8,
    GL_COEFF = 0x0A00,
    GL_DOMAIN = 0x0A02,
    GL_ORDER = 0x0A01,
    GL_FOG_HINT = 0x0C54,
    GL_LINE_SMOOTH_HINT = 0x0C52,
    GL_PERSPECTIVE_CORRECTION_HINT = 0x0C50,
    GL_POINT_SMOOTH_HINT = 0x0C51,
    GL_POLYGON_SMOOTH_HINT = 0x0C53,
    GL_DONT_CARE = 0x1100,
    GL_FASTEST = 0x1101,
    GL_NICEST = 0x1102,
    GL_SCISSOR_TEST = 0x0C11,
    GL_SCISSOR_BOX = 0x0C10,
    GL_MAP_COLOR = 0x0D10,
    GL_MAP_STENCIL = 0x0D11,
    GL_INDEX_SHIFT = 0x0D12,
    GL_INDEX_OFFSET = 0x0D13,
    GL_RED_SCALE = 0x0D14,
    GL_RED_BIAS = 0x0D15,
    GL_GREEN_SCALE = 0x0D18,
    GL_GREEN_BIAS = 0x0D19,
    GL_BLUE_SCALE = 0x0D1A,
    GL_BLUE_BIAS = 0x0D1B,
    GL_ALPHA_SCALE = 0x0D1C,
    GL_ALPHA_BIAS = 0x0D1D,
    GL_DEPTH_SCALE = 0x0D1E,
    GL_DEPTH_BIAS = 0x0D1F,
    GL_PIXEL_MAP_S_TO_S_SIZE = 0x0CB1,
    GL_PIXEL_MAP_I_TO_I_SIZE = 0x0CB0,
    GL_PIXEL_MAP_I_TO_R_SIZE = 0x0CB2,
    GL_PIXEL_MAP_I_TO_G_SIZE = 0x0CB3,
    GL_PIXEL_MAP_I_TO_B_SIZE = 0x0CB4,
    GL_PIXEL_MAP_I_TO_A_SIZE = 0x0CB5,
    GL_PIXEL_MAP_R_TO_R_SIZE = 0x0CB6,
    GL_PIXEL_MAP_G_TO_G_SIZE = 0x0CB7,
    GL_PIXEL_MAP_B_TO_B_SIZE = 0x0CB8,
    GL_PIXEL_MAP_A_TO_A_SIZE = 0x0CB9,
    GL_PIXEL_MAP_S_TO_S = 0x0C71,
    GL_PIXEL_MAP_I_TO_I = 0x0C70,
    GL_PIXEL_MAP_I_TO_R = 0x0C72,
    GL_PIXEL_MAP_I_TO_G = 0x0C73,
    GL_PIXEL_MAP_I_TO_B = 0x0C74,
    GL_PIXEL_MAP_I_TO_A = 0x0C75,
    GL_PIXEL_MAP_R_TO_R = 0x0C76,
    GL_PIXEL_MAP_G_TO_G = 0x0C77,
    GL_PIXEL_MAP_B_TO_B = 0x0C78,
    GL_PIXEL_MAP_A_TO_A = 0x0C79,
    GL_PACK_ALIGNMENT = 0x0D05,
    GL_PACK_LSB_FIRST = 0x0D01,
    GL_PACK_ROW_LENGTH = 0x0D02,
    GL_PACK_SKIP_PIXELS = 0x0D04,
    GL_PACK_SKIP_ROWS = 0x0D03,
    GL_PACK_SWAP_BYTES = 0x0D00,
    GL_UNPACK_ALIGNMENT = 0x0CF5,
    GL_UNPACK_LSB_FIRST = 0x0CF1,
    GL_UNPACK_ROW_LENGTH = 0x0CF2,
    GL_UNPACK_SKIP_PIXELS = 0x0CF4,
    GL_UNPACK_SKIP_ROWS = 0x0CF3,
    GL_UNPACK_SWAP_BYTES = 0x0CF0,
    GL_ZOOM_X = 0x0D16,
    GL_ZOOM_Y = 0x0D17,
    GL_TEXTURE_ENV = 0x2300,
    GL_TEXTURE_ENV_MODE = 0x2200,
    GL_TEXTURE_1D = 0x0DE0,
    GL_TEXTURE_2D = 0x0DE1,
    GL_TEXTURE_WRAP_S = 0x2802,
    GL_TEXTURE_WRAP_T = 0x2803,
    GL_TEXTURE_MAG_FILTER = 0x2800,
    GL_TEXTURE_MIN_FILTER = 0x2801,
    GL_TEXTURE_ENV_COLOR = 0x2201,
    GL_TEXTURE_GEN_S = 0x0C60,
    GL_TEXTURE_GEN_T = 0x0C61,
    GL_TEXTURE_GEN_MODE = 0x2500,
    GL_TEXTURE_BORDER_COLOR = 0x1004,
    GL_TEXTURE_WIDTH = 0x1000,
    GL_TEXTURE_HEIGHT = 0x1001,
    GL_TEXTURE_BORDER = 0x1005,
    GL_TEXTURE_COMPONENTS = 0x1003,
    GL_TEXTURE_RED_SIZE = 0x805C,
    GL_TEXTURE_GREEN_SIZE = 0x805D,
    GL_TEXTURE_BLUE_SIZE = 0x805E,
    GL_TEXTURE_ALPHA_SIZE = 0x805F,
    GL_TEXTURE_LUMINANCE_SIZE = 0x8060,
    GL_TEXTURE_INTENSITY_SIZE = 0x8061,
    GL_NEAREST_MIPMAP_NEAREST = 0x2700,
    GL_NEAREST_MIPMAP_LINEAR = 0x2702,
    GL_LINEAR_MIPMAP_NEAREST = 0x2701,
    GL_LINEAR_MIPMAP_LINEAR = 0x2703,
    GL_OBJECT_LINEAR = 0x2401,
    GL_OBJECT_PLANE = 0x2501,
    GL_EYE_LINEAR = 0x2400,
    GL_EYE_PLANE = 0x2502,
    GL_SPHERE_MAP = 0x2402,
    GL_DECAL = 0x2101,
    GL_MODULATE = 0x2100,
    GL_NEAREST = 0x2600,
    GL_REPEAT = 0x2901,
    GL_CLAMP = 0x2900,
    GL_S = 0x2000,
    GL_T = 0x2001,
    GL_R = 0x2002,
    GL_Q = 0x2003,
    GL_TEXTURE_GEN_R = 0x0C62,
    GL_TEXTURE_GEN_Q = 0x0C63,
    GL_PROXY_TEXTURE_1D = 0x8063,
    GL_PROXY_TEXTURE_2D = 0x8064,
    GL_TEXTURE_PRIORITY = 0x8066,
    GL_TEXTURE_RESIDENT = 0x8067,
    GL_TEXTURE_BINDING_1D = 0x8068,
    GL_TEXTURE_BINDING_2D = 0x8069,
    GL_TEXTURE_INTERNAL_FORMAT = 0x1003,
    GL_PACK_SKIP_IMAGES = 0x806B,
    GL_PACK_IMAGE_HEIGHT = 0x806C,
    GL_UNPACK_SKIP_IMAGES = 0x806D,
    GL_UNPACK_IMAGE_HEIGHT = 0x806E,
    GL_TEXTURE_3D = 0x806F,
    GL_PROXY_TEXTURE_3D = 0x8070,
    GL_TEXTURE_DEPTH = 0x8071,
    GL_TEXTURE_WRAP_R = 0x8072,
    GL_MAX_3D_TEXTURE_SIZE = 0x8073,
    GL_TEXTURE_BINDING_3D = 0x806A,
    GL_ALPHA4 = 0x803B,
    GL_ALPHA8 = 0x803C,
    GL_ALPHA12 = 0x803D,
    GL_ALPHA16 = 0x803E,
    GL_LUMINANCE4 = 0x803F,
    GL_LUMINANCE8 = 0x8040,
    GL_LUMINANCE12 = 0x8041,
    GL_LUMINANCE16 = 0x8042,
    GL_LUMINANCE4_ALPHA4 = 0x8043,
    GL_LUMINANCE6_ALPHA2 = 0x8044,
    GL_LUMINANCE8_ALPHA8 = 0x8045,
    GL_LUMINANCE12_ALPHA4 = 0x8046,
    GL_LUMINANCE12_ALPHA12 = 0x8047,
    GL_LUMINANCE16_ALPHA16 = 0x8048,
    GL_INTENSITY = 0x8049,
    GL_INTENSITY4 = 0x804A,
    GL_INTENSITY8 = 0x804B,
    GL_INTENSITY12 = 0x804C,
    GL_INTENSITY16 = 0x804D,
    GL_R3_G3_B2 = 0x2A10,
    GL_RGB4 = 0x804F,
    GL_RGB5 = 0x8050,
    GL_RGB8 = 0x8051,
    GL_RGB10 = 0x8052,
    GL_RGB12 = 0x8053,
    GL_RGB16 = 0x8054,
    GL_RGBA2 = 0x8055,
    GL_RGBA4 = 0x8056,
    GL_RGB5_A1 = 0x8057,
    GL_RGBA8 = 0x8058,
    GL_RGB10_A2 = 0x8059,
    GL_RGBA12 = 0x805A,
    GL_RGBA16 = 0x805B,
    GL_VENDOR = 0x1F00,
    GL_RENDERER = 0x1F01,
    GL_VERSION = 0x1F02,
    GL_EXTENSIONS = 0x1F03,
    GL_NO_ERROR = 0x0,
    GL_INVALID_VALUE = 0x0501,
    GL_INVALID_ENUM = 0x0500,
    GL_INVALID_OPERATION = 0x0502,
    GL_STACK_OVERFLOW = 0x0503,
    GL_STACK_UNDERFLOW = 0x0504,
    GL_OUT_OF_MEMORY = 0x0505,
    GL_RESCALE_NORMAL = 0x803A,
    GL_CLAMP_TO_EDGE = 0x812F,
    GL_MAX_ELEMENTS_VERTICES = 0x80E8,
    GL_MAX_ELEMENTS_INDICES = 0x80E9,
    GL_BGR = 0x80E0,
    GL_BGRA = 0x80E1,
    GL_UNSIGNED_BYTE_3_3_2 = 0x8032,
    GL_UNSIGNED_BYTE_2_3_3_REV = 0x8362,
    GL_UNSIGNED_SHORT_5_6_5 = 0x8363,
    GL_UNSIGNED_SHORT_5_6_5_REV = 0x8364,
    GL_UNSIGNED_SHORT_4_4_4_4 = 0x8033,
    GL_UNSIGNED_SHORT_4_4_4_4_REV = 0x8365,
    GL_UNSIGNED_SHORT_5_5_5_1 = 0x8034,
    GL_UNSIGNED_SHORT_1_5_5_5_REV = 0x8366,
    GL_UNSIGNED_INT_8_8_8_8 = 0x8035,
    GL_UNSIGNED_INT_8_8_8_8_REV = 0x8367,
    GL_UNSIGNED_INT_10_10_10_2 = 0x8036,
    GL_UNSIGNED_INT_2_10_10_10_REV = 0x8368,
    GL_LIGHT_MODEL_COLOR_CONTROL = 0x81F8,
    GL_SINGLE_COLOR = 0x81F9,
    GL_SEPARATE_SPECULAR_COLOR = 0x81FA,
    GL_TEXTURE_MIN_LOD = 0x813A,
    GL_TEXTURE_MAX_LOD = 0x813B,
    GL_TEXTURE_BASE_LEVEL = 0x813C,
    GL_TEXTURE_MAX_LEVEL = 0x813D,
    GL_SMOOTH_POINT_SIZE_RANGE = 0x0B12,
    GL_SMOOTH_POINT_SIZE_GRANULARITY = 0x0B13,
    GL_SMOOTH_LINE_WIDTH_RANGE = 0x0B22,
    GL_SMOOTH_LINE_WIDTH_GRANULARITY = 0x0B23,
    GL_ALIASED_POINT_SIZE_RANGE = 0x846D,
    GL_ALIASED_LINE_WIDTH_RANGE = 0x846E,
    GL_COLOR_TABLE = 0x80D0,
    GL_POST_CONVOLUTION_COLOR_TABLE = 0x80D1,
    GL_POST_COLOR_MATRIX_COLOR_TABLE = 0x80D2,
    GL_PROXY_COLOR_TABLE = 0x80D3,
    GL_PROXY_POST_CONVOLUTION_COLOR_TABLE = 0x80D4,
    GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE = 0x80D5,
    GL_COLOR_TABLE_SCALE = 0x80D6,
    GL_COLOR_TABLE_BIAS = 0x80D7,
    GL_COLOR_TABLE_FORMAT = 0x80D8,
    GL_COLOR_TABLE_WIDTH = 0x80D9,
    GL_COLOR_TABLE_RED_SIZE = 0x80DA,
    GL_COLOR_TABLE_GREEN_SIZE = 0x80DB,
    GL_COLOR_TABLE_BLUE_SIZE = 0x80DC,
    GL_COLOR_TABLE_ALPHA_SIZE = 0x80DD,
    GL_COLOR_TABLE_LUMINANCE_SIZE = 0x80DE,
    GL_COLOR_TABLE_INTENSITY_SIZE = 0x80DF,
    GL_CONVOLUTION_1D = 0x8010,
    GL_CONVOLUTION_2D = 0x8011,
    GL_SEPARABLE_2D = 0x8012,
    GL_CONVOLUTION_BORDER_MODE = 0x8013,
    GL_CONVOLUTION_FILTER_SCALE = 0x8014,
    GL_CONVOLUTION_FILTER_BIAS = 0x8015,
    GL_REDUCE = 0x8016,
    GL_CONVOLUTION_FORMAT = 0x8017,
    GL_CONVOLUTION_WIDTH = 0x8018,
    GL_CONVOLUTION_HEIGHT = 0x8019,
    GL_MAX_CONVOLUTION_WIDTH = 0x801A,
    GL_MAX_CONVOLUTION_HEIGHT = 0x801B,
    GL_POST_CONVOLUTION_RED_SCALE = 0x801C,
    GL_POST_CONVOLUTION_GREEN_SCALE = 0x801D,
    GL_POST_CONVOLUTION_BLUE_SCALE = 0x801E,
    GL_POST_CONVOLUTION_ALPHA_SCALE = 0x801F,
    GL_POST_CONVOLUTION_RED_BIAS = 0x8020,
    GL_POST_CONVOLUTION_GREEN_BIAS = 0x8021,
    GL_POST_CONVOLUTION_BLUE_BIAS = 0x8022,
    GL_POST_CONVOLUTION_ALPHA_BIAS = 0x8023,
    GL_CONSTANT_BORDER = 0x8151,
    GL_REPLICATE_BORDER = 0x8153,
    GL_CONVOLUTION_BORDER_COLOR = 0x8154,
    GL_COLOR_MATRIX = 0x80B1,
    GL_COLOR_MATRIX_STACK_DEPTH = 0x80B2,
    GL_MAX_COLOR_MATRIX_STACK_DEPTH = 0x80B3,
    GL_POST_COLOR_MATRIX_RED_SCALE = 0x80B4,
    GL_POST_COLOR_MATRIX_GREEN_SCALE = 0x80B5,
    GL_POST_COLOR_MATRIX_BLUE_SCALE = 0x80B6,
    GL_POST_COLOR_MATRIX_ALPHA_SCALE = 0x80B7,
    GL_POST_COLOR_MATRIX_RED_BIAS = 0x80B8,
    GL_POST_COLOR_MATRIX_GREEN_BIAS = 0x80B9,
    GL_POST_COLOR_MATRIX_BLUE_BIAS = 0x80BA,
    GL_POST_COLOR_MATRIX_ALPHA_BIAS = 0x80BB,
    GL_HISTOGRAM = 0x8024,
    GL_PROXY_HISTOGRAM = 0x8025,
    GL_HISTOGRAM_WIDTH = 0x8026,
    GL_HISTOGRAM_FORMAT = 0x8027,
    GL_HISTOGRAM_RED_SIZE = 0x8028,
    GL_HISTOGRAM_GREEN_SIZE = 0x8029,
    GL_HISTOGRAM_BLUE_SIZE = 0x802A,
    GL_HISTOGRAM_ALPHA_SIZE = 0x802B,
    GL_HISTOGRAM_LUMINANCE_SIZE = 0x802C,
    GL_HISTOGRAM_SINK = 0x802D,
    GL_MINMAX = 0x802E,
    GL_MINMAX_FORMAT = 0x802F,
    GL_MINMAX_SINK = 0x8030,
    GL_TABLE_TOO_LARGE = 0x8031,
    GL_BLEND_EQUATION = 0x8009,
    GL_MIN = 0x8007,
    GL_MAX = 0x8008,
    GL_FUNC_ADD = 0x8006,
    GL_FUNC_SUBTRACT = 0x800A,
    GL_FUNC_REVERSE_SUBTRACT = 0x800B,
    GL_BLEND_COLOR = 0x8005,
    GL_CURRENT_BIT = 0x00000001,
    GL_POINT_BIT = 0x00000002,
    GL_LINE_BIT = 0x00000004,
    GL_POLYGON_BIT = 0x00000008,
    GL_POLYGON_STIPPLE_BIT = 0x00000010,
    GL_PIXEL_MODE_BIT = 0x00000020,
    GL_LIGHTING_BIT = 0x00000040,
    GL_FOG_BIT = 0x00000080,
    GL_DEPTH_BUFFER_BIT = 0x00000100,
    GL_ACCUM_BUFFER_BIT = 0x00000200,
    GL_STENCIL_BUFFER_BIT = 0x00000400,
    GL_VIEWPORT_BIT = 0x00000800,
    GL_TRANSFORM_BIT = 0x00001000,
    GL_ENABLE_BIT = 0x00002000,
    GL_COLOR_BUFFER_BIT = 0x00004000,
    GL_HINT_BIT = 0x00008000,
    GL_EVAL_BIT = 0x00010000,
    GL_LIST_BIT = 0x00020000,
    GL_TEXTURE_BIT = 0x00040000,
    GL_SCISSOR_BIT = 0x00080000,
    GL_ALL_ATTRIB_BITS = 0x000FFFFF,
    GL_CLIENT_PIXEL_STORE_BIT = 0x00000001,
    GL_CLIENT_VERTEX_ARRAY_BIT = 0x00000002,
    GL_ALL_CLIENT_ATTRIB_BITS = 0xFFFFFFFF,
    GL_ARB_multitexture = 1,
    GL_TEXTURE0_ARB = 0x84C0,
    GL_TEXTURE1_ARB = 0x84C1,
    GL_TEXTURE2_ARB = 0x84C2,
    GL_TEXTURE3_ARB = 0x84C3,
    GL_TEXTURE4_ARB = 0x84C4,
    GL_TEXTURE5_ARB = 0x84C5,
    GL_TEXTURE6_ARB = 0x84C6,
    GL_TEXTURE7_ARB = 0x84C7,
    GL_TEXTURE8_ARB = 0x84C8,
    GL_TEXTURE9_ARB = 0x84C9,
    GL_TEXTURE10_ARB = 0x84CA,
    GL_TEXTURE11_ARB = 0x84CB,
    GL_TEXTURE12_ARB = 0x84CC,
    GL_TEXTURE13_ARB = 0x84CD,
    GL_TEXTURE14_ARB = 0x84CE,
    GL_TEXTURE15_ARB = 0x84CF,
    GL_TEXTURE16_ARB = 0x84D0,
    GL_TEXTURE17_ARB = 0x84D1,
    GL_TEXTURE18_ARB = 0x84D2,
    GL_TEXTURE19_ARB = 0x84D3,
    GL_TEXTURE20_ARB = 0x84D4,
    GL_TEXTURE21_ARB = 0x84D5,
    GL_TEXTURE22_ARB = 0x84D6,
    GL_TEXTURE23_ARB = 0x84D7,
    GL_TEXTURE24_ARB = 0x84D8,
    GL_TEXTURE25_ARB = 0x84D9,
    GL_TEXTURE26_ARB = 0x84DA,
    GL_TEXTURE27_ARB = 0x84DB,
    GL_TEXTURE28_ARB = 0x84DC,
    GL_TEXTURE29_ARB = 0x84DD,
    GL_TEXTURE30_ARB = 0x84DE,
    GL_TEXTURE31_ARB = 0x84DF,
    GL_ACTIVE_TEXTURE_ARB = 0x84E0,
    GL_CLIENT_ACTIVE_TEXTURE_ARB = 0x84E1,
    GL_MAX_TEXTURE_UNITS_ARB = 0x84E2,
    GL_EXT_abgr = 1,
    GL_ABGR_EXT = 0x8000,
    GL_EXT_blend_color = 1,
    GL_CONSTANT_COLOR_EXT = 0x8001,
    GL_ONE_MINUS_CONSTANT_COLOR_EXT = 0x8002,
    GL_CONSTANT_ALPHA_EXT = 0x8003,
    GL_ONE_MINUS_CONSTANT_ALPHA_EXT = 0x8004,
    GL_BLEND_COLOR_EXT = 0x8005,
    GL_EXT_polygon_offset = 1,
    GL_POLYGON_OFFSET_EXT = 0x8037,
    GL_POLYGON_OFFSET_FACTOR_EXT = 0x8038,
    GL_POLYGON_OFFSET_BIAS_EXT = 0x8039,
    GL_EXT_texture3D = 1,
    GL_PACK_SKIP_IMAGES_EXT = 0x806B,
    GL_PACK_IMAGE_HEIGHT_EXT = 0x806C,
    GL_UNPACK_SKIP_IMAGES_EXT = 0x806D,
    GL_UNPACK_IMAGE_HEIGHT_EXT = 0x806E,
    GL_TEXTURE_3D_EXT = 0x806F,
    GL_PROXY_TEXTURE_3D_EXT = 0x8070,
    GL_TEXTURE_DEPTH_EXT = 0x8071,
    GL_TEXTURE_WRAP_R_EXT = 0x8072,
    GL_MAX_3D_TEXTURE_SIZE_EXT = 0x8073,
    GL_TEXTURE_3D_BINDING_EXT = 0x806A,
    GL_EXT_texture_object = 1,
    GL_TEXTURE_PRIORITY_EXT = 0x8066,
    GL_TEXTURE_RESIDENT_EXT = 0x8067,
    GL_TEXTURE_1D_BINDING_EXT = 0x8068,
    GL_TEXTURE_2D_BINDING_EXT = 0x8069,
    GL_EXT_rescale_normal = 1,
    GL_RESCALE_NORMAL_EXT = 0x803A,
    GL_EXT_vertex_array = 1,
    GL_VERTEX_ARRAY_EXT = 0x8074,
    GL_NORMAL_ARRAY_EXT = 0x8075,
    GL_COLOR_ARRAY_EXT = 0x8076,
    GL_INDEX_ARRAY_EXT = 0x8077,
    GL_TEXTURE_COORD_ARRAY_EXT = 0x8078,
    GL_EDGE_FLAG_ARRAY_EXT = 0x8079,
    GL_VERTEX_ARRAY_SIZE_EXT = 0x807A,
    GL_VERTEX_ARRAY_TYPE_EXT = 0x807B,
    GL_VERTEX_ARRAY_STRIDE_EXT = 0x807C,
    GL_VERTEX_ARRAY_COUNT_EXT = 0x807D,
    GL_NORMAL_ARRAY_TYPE_EXT = 0x807E,
    GL_NORMAL_ARRAY_STRIDE_EXT = 0x807F,
    GL_NORMAL_ARRAY_COUNT_EXT = 0x8080,
    GL_COLOR_ARRAY_SIZE_EXT = 0x8081,
    GL_COLOR_ARRAY_TYPE_EXT = 0x8082,
    GL_COLOR_ARRAY_STRIDE_EXT = 0x8083,
    GL_COLOR_ARRAY_COUNT_EXT = 0x8084,
    GL_INDEX_ARRAY_TYPE_EXT = 0x8085,
    GL_INDEX_ARRAY_STRIDE_EXT = 0x8086,
    GL_INDEX_ARRAY_COUNT_EXT = 0x8087,
    GL_TEXTURE_COORD_ARRAY_SIZE_EXT = 0x8088,
    GL_TEXTURE_COORD_ARRAY_TYPE_EXT = 0x8089,
    GL_TEXTURE_COORD_ARRAY_STRIDE_EXT = 0x808A,
    GL_TEXTURE_COORD_ARRAY_COUNT_EXT = 0x808B,
    GL_EDGE_FLAG_ARRAY_STRIDE_EXT = 0x808C,
    GL_EDGE_FLAG_ARRAY_COUNT_EXT = 0x808D,
    GL_VERTEX_ARRAY_POINTER_EXT = 0x808E,
    GL_NORMAL_ARRAY_POINTER_EXT = 0x808F,
    GL_COLOR_ARRAY_POINTER_EXT = 0x8090,
    GL_INDEX_ARRAY_POINTER_EXT = 0x8091,
    GL_TEXTURE_COORD_ARRAY_POINTER_EXT = 0x8092,
    GL_EDGE_FLAG_ARRAY_POINTER_EXT = 0x8093,
    GL_SGIS_texture_edge_clamp = 1,
    GL_CLAMP_TO_EDGE_SGIS = 0x812F,
    GL_EXT_blend_minmax = 1,
    GL_FUNC_ADD_EXT = 0x8006,
    GL_MIN_EXT = 0x8007,
    GL_MAX_EXT = 0x8008,
    GL_BLEND_EQUATION_EXT = 0x8009,
    GL_EXT_blend_subtract = 1,
    GL_FUNC_SUBTRACT_EXT = 0x800A,
    GL_FUNC_REVERSE_SUBTRACT_EXT = 0x800B,
    GL_EXT_blend_logic_op = 1,
    GL_EXT_point_parameters = 1,
    GL_POINT_SIZE_MIN_EXT = 0x8126,
    GL_POINT_SIZE_MAX_EXT = 0x8127,
    GL_POINT_FADE_THRESHOLD_SIZE_EXT = 0x8128,
    GL_DISTANCE_ATTENUATION_EXT = 0x8129,
    GL_EXT_paletted_texture = 1,
    GL_TABLE_TOO_LARGE_EXT = 0x8031,
    GL_COLOR_TABLE_FORMAT_EXT = 0x80D8,
    GL_COLOR_TABLE_WIDTH_EXT = 0x80D9,
    GL_COLOR_TABLE_RED_SIZE_EXT = 0x80DA,
    GL_COLOR_TABLE_GREEN_SIZE_EXT = 0x80DB,
    GL_COLOR_TABLE_BLUE_SIZE_EXT = 0x80DC,
    GL_COLOR_TABLE_ALPHA_SIZE_EXT = 0x80DD,
    GL_COLOR_TABLE_LUMINANCE_SIZE_EXT = 0x80DE,
    GL_COLOR_TABLE_INTENSITY_SIZE_EXT = 0x80DF,
    GL_TEXTURE_INDEX_SIZE_EXT = 0x80ED,
    GL_COLOR_INDEX1_EXT = 0x80E2,
    GL_COLOR_INDEX2_EXT = 0x80E3,
    GL_COLOR_INDEX4_EXT = 0x80E4,
    GL_COLOR_INDEX8_EXT = 0x80E5,
    GL_COLOR_INDEX12_EXT = 0x80E6,
    GL_COLOR_INDEX16_EXT = 0x80E7,
    GL_EXT_clip_volume_hint = 1,
    GL_CLIP_VOLUME_CLIPPING_HINT_EXT = 0x80F0,
    GL_EXT_compiled_vertex_array = 1,
    GL_ARRAY_ELEMENT_LOCK_FIRST_EXT = 0x81A8,
    GL_ARRAY_ELEMENT_LOCK_COUNT_EXT = 0x81A9,
    GL_HP_occlusion_test = 1,
    GL_OCCLUSION_TEST_HP = 0x8165,
    GL_OCCLUSION_TEST_RESULT_HP = 0x8166,
    GL_EXT_shared_texture_palette = 1,
    GL_SHARED_TEXTURE_PALETTE_EXT = 0x81FB,
    GL_EXT_stencil_wrap = 1,
    GL_INCR_WRAP_EXT = 0x8507,
    GL_DECR_WRAP_EXT = 0x8508,
    GL_NV_texgen_reflection = 1,
    GL_NORMAL_MAP_NV = 0x8511,
    GL_REFLECTION_MAP_NV = 0x8512,
    GL_EXT_texture_env_add = 1,
    GL_MESA_window_pos = 1,
    GL_MESA_resize_buffers = 1,
    GL_EXT_texture_env_dot3 = 1,
    GL_DOT3_RGB_EXT = 0x8740,
    GL_DOT3_RGBA_EXT = 0x8741
  };
%}

