/***************************************************************************
                             util.cc  -  Utility functions
                               -------------------
      copyright            : (C) 2003 by Morten Hustveit
      email                : morten@debian.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <sys/stat.h>
#include <sys/types.h>

#include <espace/api.h>
#include <espace/console.h>
#include <espace/file.h>
#include <espace/opengl.h>
#include <espace/plugins.h>
#include <espace/sound.h>
#include <espace/shader.h>
#include <espace/string.h>
#include <espace/system.h>
#include <espace/util.h>

namespace
{
  uint flags = 0;
}

void Util::initialize(int argc, char** argv, uint flags)
{
  Plugin::initialize();

  API::initialize();

  if(argc)
  {
    for(int i = 1; i < argc; ++i)
    {
      if(argv[i][0] == '+')
      {
        String command = argv[i] + 1;

        while(i < argc - 1 && argv[i + 1][0] != '+')
        {
          command += " ";
          command += argv[++i];
        }

        API::executeCommand(command);
      }
    }
  }

  System::initialize();

  GL::initialize();

  if(!(flags & NoAudio))
    Sound::initialize();

  if(!(flags & NoShaders))
  {
    Shader::initialize();
    Shader::setDefaults();
  }

  ::flags = flags;
}

void Util::idle()
{
  if(!(flags & NoConsole))
    Console::render();

  if(!(flags & NoAudio))
    Sound::idle();

  System::idle();

  API::flushCommands();

  Shader::time = System::time();
}

